# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: digiwave/v1/digiwave.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1a\x64igiwave/v1/digiwave.proto\x12\x08\x64igiwave\"K\n\x10\x43onfigurationSet\x12\x13\n\x0b\x66ull_duplex\x18\x01 \x01(\x08\x12\x10\n\x08\x63laim_rx\x18\x02 \x01(\x08\x12\x10\n\x08loopback\x18\x03 \x01(\x08\"\x1a\n\x18\x43onfigurationSetResponse\"\x12\n\x10\x43onfigurationGet\"\x1a\n\x18\x43onfigurationGetResponse\"\x17\n\x15\x43onfigurationDescribe\"\x1f\n\x1d\x43onfigurationDescribeResponse\"\x14\n\x12\x46unctionControlGet\"\"\n\x12\x46unctionControlSet\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"\x1c\n\x1a\x46unctionControlGetResponse\"\x1c\n\x1a\x46unctionControlSetResponse\"\x14\n\x12StreamControlStart\"#\n\x06Sample\x12\x19\n\x11transitions_block\x18\x01 \x01(\x0c\"/\n\nStreamData\x12!\n\x07samples\x18\x01 \x03(\x0b\x32\x10.digiwave.SampleB\rZ\x0b\x64igiwave/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'digiwave.v1.digiwave_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\013digiwave/v1'
  _globals['_CONFIGURATIONSET']._serialized_start=40
  _globals['_CONFIGURATIONSET']._serialized_end=115
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_start=117
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_end=143
  _globals['_CONFIGURATIONGET']._serialized_start=145
  _globals['_CONFIGURATIONGET']._serialized_end=163
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_start=165
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_end=191
  _globals['_CONFIGURATIONDESCRIBE']._serialized_start=193
  _globals['_CONFIGURATIONDESCRIBE']._serialized_end=216
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_start=218
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_end=249
  _globals['_FUNCTIONCONTROLGET']._serialized_start=251
  _globals['_FUNCTIONCONTROLGET']._serialized_end=271
  _globals['_FUNCTIONCONTROLSET']._serialized_start=273
  _globals['_FUNCTIONCONTROLSET']._serialized_end=307
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_start=309
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_end=337
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_start=339
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_end=367
  _globals['_STREAMCONTROLSTART']._serialized_start=369
  _globals['_STREAMCONTROLSTART']._serialized_end=389
  _globals['_SAMPLE']._serialized_start=391
  _globals['_SAMPLE']._serialized_end=426
  _globals['_STREAMDATA']._serialized_start=428
  _globals['_STREAMDATA']._serialized_end=475
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: bitbusSniffer/v1/bitbusSniffer.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$bitbusSniffer/v1/bitbusSniffer.proto\x12\rbitbusSniffer\"R\n\x10\x43onfigurationSet\x12\x12\n\nignore_crc\x18\x01 \x01(\x08\x12\x12\n\nbaud_62500\x18\x02 \x01(\x08\x12\x16\n\x0e\x61\x64\x64ress_filter\x18\x03 \x01(\x0c\"\x1a\n\x18\x43onfigurationSetResponse\"\x12\n\x10\x43onfigurationGet\"\x1a\n\x18\x43onfigurationGetResponse\"\x17\n\x15\x43onfigurationDescribe\"\x1f\n\x1d\x43onfigurationDescribeResponse\"\x14\n\x12\x46unctionControlGet\"\x1d\n\x05\x46rame\x12\x14\n\x0c\x62itbus_frame\x18\x01 \x01(\x0c\":\n\x12\x46unctionControlSet\x12$\n\x06\x66rames\x18\x01 \x03(\x0b\x32\x14.bitbusSniffer.Frame\"\x1c\n\x1a\x46unctionControlGetResponse\"\x1c\n\x1a\x46unctionControlSetResponse\"\x14\n\x12StreamControlStart\"\x82\x01\n\x06Sample\x12\x11\n\ttimestamp\x18\x01 \x01(\x06\x12\r\n\x05\x66lags\x18\x02 \x01(\r\x12\x14\n\x0c\x62itbus_frame\x18\x03 \x01(\x0c\"@\n\x05\x46lags\x12\x08\n\x04none\x10\x00\x12\x0b\n\x07\x62\x61\x64_crc\x10\x01\x12\x0f\n\x0b\x66rames_lost\x10\x10\x12\x0f\n\x0b\x62uf_overrun\x10 \"4\n\nStreamData\x12&\n\x07samples\x18\x01 \x03(\x0b\x32\x15.bitbusSniffer.SampleB\x12Z\x10\x62itbusSniffer/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'bitbusSniffer.v1.bitbusSniffer_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\020bitbusSniffer/v1'
  _globals['_CONFIGURATIONSET']._serialized_start=55
  _globals['_CONFIGURATIONSET']._serialized_end=137
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_start=139
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_end=165
  _globals['_CONFIGURATIONGET']._serialized_start=167
  _globals['_CONFIGURATIONGET']._serialized_end=185
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_start=187
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_end=213
  _globals['_CONFIGURATIONDESCRIBE']._serialized_start=215
  _globals['_CONFIGURATIONDESCRIBE']._serialized_end=238
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_start=240
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_end=271
  _globals['_FUNCTIONCONTROLGET']._serialized_start=273
  _globals['_FUNCTIONCONTROLGET']._serialized_end=293
  _globals['_FRAME']._serialized_start=295
  _globals['_FRAME']._serialized_end=324
  _globals['_FUNCTIONCONTROLSET']._serialized_start=326
  _globals['_FUNCTIONCONTROLSET']._serialized_end=384
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_start=386
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_end=414
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_start=416
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_end=444
  _globals['_STREAMCONTROLSTART']._serialized_start=446
  _globals['_STREAMCONTROLSTART']._serialized_end=466
  _globals['_SAMPLE']._serialized_start=469
  _globals['_SAMPLE']._serialized_end=599
  _globals['_SAMPLE_FLAGS']._serialized_start=535
  _globals['_SAMPLE_FLAGS']._serialized_end=599
  _globals['_STREAMDATA']._serialized_start=601
  _globals['_STREAMDATA']._serialized_end=653
# @@protoc_insertion_point(module_scope)



# SPDX-License-Identifer: Apache-2.0
from io4edge_client.functionblock import Client as FbClient
import io4edge_client.api.binaryIoTypeD.python.binaryIoTypeD.v1.binaryIoTypeD_pb2 as Pb
import io4edge_client.api.io4edge.python.functionblock.v1alpha1.io4edge_functionblock_pb2 as FbPb

class Client:
    """
    binaryIoTypeD functionblock client.
    @param addr: address of io4edge function block (mdns name or "ip:port" address)
    @param command_timeout: timeout for commands in seconds
    """

    def __init__(self, addr: str, command_timeout=5):
        self._fb_client = FbClient("_io4edge_binaryIoTypeD._tcp", addr, command_timeout)

    def upload_configuration(self, config: Pb.ConfigurationSet):
        """
        Upload the configuration to the binaryIoTypeD functionblock.
        @param config: configuration to upload
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        self._fb_client.upload_configuration(config)

    def download_configuration(self) -> Pb.ConfigurationGetResponse:
        """
        Download the configuration from the binaryIoTypeD functionblock.
        @return: actual configuration
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_response = Pb.ConfigurationGetResponse()
        self._fb_client.download_configuration(Pb.ConfigurationGet(), fs_response)
        return fs_response

    def set_output(self, channel: int, state: bool):
        """
        Set the state of a single output.
        @param channel: channel number
        @param state: state to set. a "true" state turns on the output switch, a "false" state turns it off.
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_cmd = Pb.FunctionControlSet()
        fs_cmd.single.channel = channel
        fs_cmd.single.state = state
        self._fb_client.function_control_set(fs_cmd, Pb.FunctionControlSetResponse())

    def set_outputs(self, states: int, mask: int):
        """
        Set the state of all or a group of output channels.
        @param states: binary coded map of outputs. 0 means switch off, 1 means switch on, LSB is Channel0
        @param mask: binary coded map of outputs to be set. 0 means do not change, 1 means change, LSB is Channel0
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_cmd = Pb.FunctionControlSet()
        fs_cmd.all.values = states
        fs_cmd.all.mask = mask
        self._fb_client.function_control_set(fs_cmd, Pb.FunctionControlSetResponse())

    def get_channels(self) -> Pb.FunctionControlGetResponse:
        """
        Get the state of all channels, regardless whether they are configured as input or output.
        Each bit in the returned "inputs" corresponds to one channel, bit0 being channel 0.
        The bit is false if the pin level is inactive, or true if active.
        diag is a slice with bitfields containing diagnostic bits for each channel.
        @return: state of all inputs plus diagnostic info
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_cmd = Pb.FunctionControlGet()
        fs_response = Pb.FunctionControlGetResponse()
        self._fb_client.function_control_get(fs_cmd, fs_response)
        return fs_response

    def close(self):
        """
        Close the connection to the function block, terminate read thread.
        After calling this method, the object is no longer usable.
        """
        self._fb_client.close()



# SPDX-License-Identifer: Apache-2.0
import threading
from collections import deque
from io4edge_client.base import Client as BaseClient
from ..util.any import pb_any_unpack
import io4edge_client.api.io4edge.python.functionblock.v1alpha1.io4edge_functionblock_pb2 as FbPb
import google.protobuf.any_pb2 as AnyPb


class Client:
    """
    io4edge functionblock client.
    @param addr: address of io4edge function block (mdns name or "ip:port" address)
    @param service: service name of io4edge function block
    @param command_timeout: timeout for commands in seconds
    """

    def __init__(self, service: str, addr: str, command_timeout=5):
        self._client = BaseClient(service, addr)
        self._stream_queue_mutex = (
            threading.Lock()
        )  # Protects _stream_queue from concurrent access
        self._stream_queue_sema = threading.Semaphore(0)  # count items in _stream_queue
        self._stream_queue = deque()
        self._cmd_event = threading.Event()
        self._cmd_mutex = (
            threading.Lock()
        )  # Ensures only one command is pending at a time
        self._cmd_response = None
        self._cmd_context = 0  # sequence number for command context
        self._cmd_timeout = command_timeout
        self._read_thread_stop = False
        self._read_thread_id = threading.Thread(target=self._read_thread, daemon=True)
        self._read_thread_id.start()

    def upload_configuration(self, fs_cmd):
        """
        Upload configuration to io4edge function block.
        @param fs_cmd: protobuf message with the function specific configuration
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_any = AnyPb.Any()
        fs_any.Pack(fs_cmd)

        fb_cmd = FbPb.Command()
        fb_cmd.Configuration.functionSpecificConfigurationSet.CopyFrom(fs_any)
        self._command(fb_cmd)

    def download_configuration(self, fs_cmd, fs_response):
        """
        Download configuration from io4edge function block.
        @param fs_cmd: protobuf message with the function specific configuration (mostly empty)
        @param fs_response: protobuf message that is filled with the function specific configuration response
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_any = AnyPb.Any()
        fs_any.Pack(fs_cmd)

        fb_cmd = FbPb.Command()
        fb_cmd.Configuration.functionSpecificConfigurationGet.CopyFrom(fs_any)
        fb_res = self._command(fb_cmd)
        pb_any_unpack(
            fb_res.Configuration.functionSpecificConfigurationGet, fs_response
        )

    def describe(self, fs_cmd, fs_response):
        """
        Describe the function block (call the firmware describe function).
        @param fs_cmd: protobuf message with the function specific describe request (mostly empty)
        @param fs_response: protobuf message that is filled with the function specific describe response
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_any = AnyPb.Any()
        fs_any.Pack(fs_cmd)

        fb_cmd = FbPb.Command()
        fb_cmd.Configuration.functionSpecificConfigurationDescribe.CopyFrom(fs_any)
        fb_res = self._command(fb_cmd)
        pb_any_unpack(
            fb_res.Configuration.functionSpecificConfigurationDescribe, fs_response
        )

    def function_control_set(self, fs_cmd, fs_response):
        """
        Execute "function control set" command on io4edge function block.
        @param fs_cmd: protobuf message with the function specific function control set request
        @param fs_response: protobuf message that is filled with the function specific function control set response
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_any = AnyPb.Any()
        fs_any.Pack(fs_cmd)

        fb_cmd = FbPb.Command()
        fb_cmd.functionControl.functionSpecificFunctionControlSet.CopyFrom(fs_any)
        fb_res = self._command(fb_cmd)
        pb_any_unpack(
            fb_res.functionControl.functionSpecificFunctionControlSet, fs_response
        )

    def function_control_get(self, fs_cmd, fs_response):
        """
        Execute "function control get" command on io4edge function block.
        @param fs_cmd: protobuf message with the function specific function control get request (mostly empty)
        @param fs_response: protobuf message that is filled with the function specific function control get response
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_any = AnyPb.Any()
        fs_any.Pack(fs_cmd)

        fb_cmd = FbPb.Command()
        fb_cmd.functionControl.functionSpecificFunctionControlGet.CopyFrom(fs_any)
        fb_res = self._command(fb_cmd)
        pb_any_unpack(
            fb_res.functionControl.functionSpecificFunctionControlGet, fs_response
        )

    def start_stream(self, fs_config, fb_config: FbPb.StreamControlStart):
        """
        Start streaming data from io4edge function block.
        @param fs_config: protobuf message with the function specific configuration
        @param fb_config: protobuf message with the function block configuration
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_any = AnyPb.Any()
        fs_any.Pack(fs_config)

        fb_config.functionSpecificStreamControlStart.CopyFrom(fs_any)
        fb_cmd = FbPb.Command()
        fb_cmd.streamControl.start.CopyFrom(fb_config)

        self._command(fb_cmd)

    def stop_stream(self):
        """
        Stop streaming data from io4edge function block.
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fb_cmd = FbPb.Command()
        stop = FbPb.StreamControlStop()
        fb_cmd.streamControl.stop.CopyFrom(stop)
        self._command(fb_cmd)

    def read_stream(self, timeout, stream_data):
        """
        Read next message from stream.
        @param timeout: timeout in seconds
        @param stream_data: protobuf message that is filled with the stream data
        @return functionblock stream meta data (deliveryTimestampUs, sequence)
        @raises TimeoutError: if no data is available within the timeout
        """
        if not self._stream_queue_sema.acquire(timeout=timeout):
            raise TimeoutError("No data available within timeout")
        with self._stream_queue_mutex:
            data = self._stream_queue.popleft()
            pb_any_unpack(data.functionSpecificStreamData, stream_data)
            return data

    def _command(self, cmd: FbPb.Command):
        with self._cmd_mutex:
            cmd.context.value = str(self._cmd_context)
            self._cmd_event.clear()
            self._client.write_msg(cmd)
            if not self._cmd_event.wait(timeout=self._cmd_timeout):
                raise TimeoutError("Command timed out")

            response = self._cmd_response
            if response.context.value != str(self._cmd_context):
                raise RuntimeError(
                    f"Command context mismatch. Got {response.context.value}, expected {self._cmd_context}"
                )

            self._cmd_context += 1

            if response.status != FbPb.Status.OK:
                status_str = FbPb.Status.Name(response.status)
                raise RuntimeError(f"Command failed: {status_str}: {response.error}")
            return response

    def _read_thread(self):
        while not self._read_thread_stop:
            msg = FbPb.Response()
            try:
                self._client.read_msg(msg, timeout=1)
            except TimeoutError:
                continue

            if msg.WhichOneof("type") == "stream":
                self._feed_stream(msg.stream)
            else:
                self._cmd_response = msg
                self._cmd_event.set()

    def _feed_stream(self, stream_data):
        with self._stream_queue_mutex:
            self._stream_queue.append(stream_data)
        self._stream_queue_sema.release()

    def close(self):
        """
        Close the connection to the function block, terminate read thread.
        After calling this method, the object is no longer usable.
        """
        self._read_thread_stop = True
        self._read_thread_id.join()
        self._client.close()



# SPDX-License-Identifer: Apache-2.0
from io4edge_client.functionblock import Client as FbClient
import io4edge_client.api.mvbSniffer.python.mvbSniffer.v1.mvbSniffer_pb2 as Pb
import io4edge_client.api.mvbSniffer.python.mvbSniffer.v1.telegram_pb2 as TelegramPb
import io4edge_client.api.io4edge.python.functionblock.v1alpha1.io4edge_functionblock_pb2 as FbPb


class Client:
    """
    mvbSniffer functionblock client.
    @param addr: address of io4edge function block (mdns name or "ip:port" address)
    @param command_timeout: timeout for commands in seconds
    """

    def __init__(self, addr: str, command_timeout=5):
        self._fb_client = FbClient("_io4edge_mvbSniffer._tcp", addr, command_timeout)

    def send_pattern(self, msg: str):
        """
        Send a pattern to the mvbSniffer's internal mvb frame generator.
        See https://github.com/ci4rail/io4edge-client-go/blob/main/mvbsniffer/generator.go how to create the pattern.
        @param msg: pattern to send
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_cmd = Pb.FunctionControlSet(generator_pattern=msg)
        self._fb_client.function_control_set(fs_cmd, Pb.FunctionControlSetResponse())

    def start_stream(
        self, config: Pb.StreamControlStart, fb_config: FbPb.StreamControl
    ):
        """
        Start streaming of mvbSniffer data.
        @param config: mvb specific filter configuration of the stream
        @param fb_config: functionblock generic configuration of the stream
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        self._fb_client.start_stream(config, fb_config)

    def stop_stream(self):
        """
        Stop streaming of mvbSniffer data.
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        self._fb_client.stop_stream()

    def read_stream(self, timeout=None):
        """
        Read the next message from the stream.
        @param timeout: timeout in seconds
        @return: functionblock generic stream data (deliveryTimestampUs, sequence), mvbSniffer TelegramCollection
        @raises TimeoutError: if no data is available within the timeout
        """
        stream_data = TelegramPb.TelegramCollection()
        generic_stream_data = self._fb_client.read_stream(timeout, stream_data)
        return generic_stream_data, stream_data

    def close(self):
        """
        Close the connection to the function block, terminate read thread.
        After calling this method, the object is no longer usable.
        """
        self._fb_client.close()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: canL2/v1alpha1/canL2.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1a\x63\x61nL2/v1alpha1/canL2.proto\x12\x05\x63\x61nL2\"V\n\x10\x43onfigurationSet\x12\x0c\n\x04\x62\x61ud\x18\x01 \x01(\x07\x12\x13\n\x0bsamplePoint\x18\x02 \x01(\x05\x12\x0b\n\x03sjw\x18\x03 \x01(\x05\x12\x12\n\nlistenOnly\x18\x04 \x01(\x08\"\x1a\n\x18\x43onfigurationSetResponse\"\x12\n\x10\x43onfigurationGet\"^\n\x18\x43onfigurationGetResponse\x12\x0c\n\x04\x62\x61ud\x18\x01 \x01(\x07\x12\x13\n\x0bsamplePoint\x18\x02 \x01(\x05\x12\x0b\n\x03sjw\x18\x03 \x01(\x05\x12\x12\n\nlistenOnly\x18\x04 \x01(\x08\"\x17\n\x15\x43onfigurationDescribe\"\x1f\n\x1d\x43onfigurationDescribeResponse\"\xb9\x01\n\x15\x43onfigurationResponse\x12.\n\x03get\x18\x01 \x01(\x0b\x32\x1f.canL2.ConfigurationGetResponseH\x00\x12.\n\x03set\x18\x02 \x01(\x0b\x32\x1f.canL2.ConfigurationSetResponseH\x00\x12\x38\n\x08\x64\x65scribe\x18\x03 \x01(\x0b\x32$.canL2.ConfigurationDescribeResponseH\x00\x42\x06\n\x04type\"\x14\n\x12\x46unctionControlGet\"Z\n\x05\x46rame\x12\x1b\n\x13\x65xtendedFrameFormat\x18\x01 \x01(\x08\x12\x13\n\x0bremoteFrame\x18\x02 \x01(\x08\x12\x11\n\tmessageId\x18\x03 \x01(\x07\x12\x0c\n\x04\x64\x61ta\x18\x04 \x01(\x0c\"1\n\x12\x46unctionControlSet\x12\x1b\n\x05\x66rame\x18\x01 \x03(\x0b\x32\x0c.canL2.Frame\"M\n\x1a\x46unctionControlGetResponse\x12/\n\x0f\x63ontrollerState\x18\x01 \x01(\x0e\x32\x16.canL2.ControllerState\"\x1c\n\x1a\x46unctionControlSetResponse\"D\n\x12StreamControlStart\x12\x16\n\x0e\x61\x63\x63\x65ptanceCode\x18\x01 \x01(\x07\x12\x16\n\x0e\x61\x63\x63\x65ptanceMask\x18\x02 \x01(\x07\"\xa0\x01\n\x06Sample\x12\x11\n\ttimestamp\x18\x01 \x01(\x06\x12\x1b\n\x05\x66rame\x18\x02 \x01(\x0b\x32\x0c.canL2.Frame\x12/\n\x0f\x63ontrollerState\x18\x03 \x01(\x0e\x32\x16.canL2.ControllerState\x12 \n\x05\x65rror\x18\x04 \x01(\x0e\x32\x11.canL2.ErrorEvent\x12\x13\n\x0bisDataFrame\x18\x05 \x01(\x08\",\n\nStreamData\x12\x1e\n\x07samples\x18\x01 \x03(\x0b\x32\r.canL2.Sample*E\n\x0f\x43ontrollerState\x12\n\n\x06\x43\x41N_OK\x10\x00\x12\x15\n\x11\x43\x41N_ERROR_PASSIVE\x10\x01\x12\x0f\n\x0b\x43\x41N_BUS_OFF\x10\x02*m\n\nErrorEvent\x12\x10\n\x0c\x43\x41N_NO_ERROR\x10\x00\x12\x11\n\rCAN_TX_FAILED\x10\x01\x12\x15\n\x11\x43\x41N_RX_QUEUE_FULL\x10\x02\x12\x10\n\x0c\x43\x41N_ARB_LOST\x10\x03\x12\x11\n\rCAN_BUS_ERROR\x10\x04\x42\x10Z\x0e\x63\x61nL2/v1alpha1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'canL2.v1alpha1.canL2_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\016canL2/v1alpha1'
  _globals['_CONTROLLERSTATE']._serialized_start=1068
  _globals['_CONTROLLERSTATE']._serialized_end=1137
  _globals['_ERROREVENT']._serialized_start=1139
  _globals['_ERROREVENT']._serialized_end=1248
  _globals['_CONFIGURATIONSET']._serialized_start=37
  _globals['_CONFIGURATIONSET']._serialized_end=123
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_start=125
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_end=151
  _globals['_CONFIGURATIONGET']._serialized_start=153
  _globals['_CONFIGURATIONGET']._serialized_end=171
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_start=173
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_end=267
  _globals['_CONFIGURATIONDESCRIBE']._serialized_start=269
  _globals['_CONFIGURATIONDESCRIBE']._serialized_end=292
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_start=294
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_end=325
  _globals['_CONFIGURATIONRESPONSE']._serialized_start=328
  _globals['_CONFIGURATIONRESPONSE']._serialized_end=513
  _globals['_FUNCTIONCONTROLGET']._serialized_start=515
  _globals['_FUNCTIONCONTROLGET']._serialized_end=535
  _globals['_FRAME']._serialized_start=537
  _globals['_FRAME']._serialized_end=627
  _globals['_FUNCTIONCONTROLSET']._serialized_start=629
  _globals['_FUNCTIONCONTROLSET']._serialized_end=678
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_start=680
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_end=757
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_start=759
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_end=787
  _globals['_STREAMCONTROLSTART']._serialized_start=789
  _globals['_STREAMCONTROLSTART']._serialized_end=857
  _globals['_SAMPLE']._serialized_start=860
  _globals['_SAMPLE']._serialized_end=1020
  _globals['_STREAMDATA']._serialized_start=1022
  _globals['_STREAMDATA']._serialized_end=1066
# @@protoc_insertion_point(module_scope)



# SPDX-License-Identifer: Apache-2.0
from .client import Client  # noqa: F401
import io4edge_client.api.mvbSniffer.python.mvbSniffer.v1.mvbSniffer_pb2 as Pb  # noqa: F401
import io4edge_client.api.mvbSniffer.python.mvbSniffer.v1.telegram_pb2 as TelegramPb  # noqa: F401



from contextlib import contextmanager
import os
import sys
from io4edge_client.base import Client as BaseClient
import io4edge_client.api.io4edge.python.core_api.v1alpha2.io4edge_core_api_pb2 as Pb
from ..types import FirmwareIdentification, HardwareIdentification
from typing import Callable, Optional


class PbCoreClient:
    """
    io4edge core client using protobuf communication.
    @param addr: address of io4edge function block (mdns name or "ip:port" address)
    @param command_timeout: timeout for commands in seconds
    """

    def __init__(self, addr: str, command_timeout=5):
        self._addr = addr
        self._command_timeout = command_timeout
        self._client: BaseClient | None = None

    @contextmanager
    def connection(self):
        """
        Context manager for the connection.
        """
        self._client = BaseClient("_io4edge-core._tcp", self._addr)
        try:
            yield
        finally:
            self._client.close()
            self._client = None

    def command(self, cmd, response):
        """
        Send a command to the io4edge core.
        @param cmd: protobuf message with the command
        @param response: protobuf message that is filled with the response
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        with self.connection():
            self._client.write_msg(cmd)
            self._client.read_msg(response, self._command_timeout)
            # Due to a bug in the io4edge core, the response ID is not set correctly in the get/set parameter response.
            if response.id != cmd.id and cmd.id != Pb.CommandId.GET_PERSISTENT_PARAMETER and cmd.id != Pb.CommandId.SET_PERSISTENT_PARAMETER:
                raise RuntimeError(
                    f"Unexpected response ID, expected {cmd.id}, got {response.id}")
            if response.status != Pb.Status.OK:
                raise RuntimeError(
                    f"Command failed with status {response.status} ({Pb.Status.Name(response.status)})")

    def identify_hardware(self) -> HardwareIdentification:
        """
        Identify the hardware of io4edge device.
        @return: hardware identification containing root article, major version, and serial number
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        cmd = Pb.CoreCommand(id=Pb.CommandId.IDENTIFY_HARDWARE)
        response = Pb.CoreResponse()
        self.command(cmd, response)
        return HardwareIdentification(
            response.identify_hardware.root_article,
            response.identify_hardware.major_version,
            response.identify_hardware.serial_number,
        )

    def program_hardware_identification(
        self, root_article: str, major_version: int, serial_number: str
    ) -> None:
        raise NotImplementedError(
            "Programming hardware identification is not implemented yet"
        )

    def identify_firmware(self) -> FirmwareIdentification:
        """
        Identify the firmware version of io4edge device.
        @return: firmware identification containing name and version
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        cmd = Pb.CoreCommand(id=Pb.CommandId.IDENTIFY_FIRMWARE)
        response = Pb.CoreResponse()
        self.command(cmd, response)
        return FirmwareIdentification(
            response.identify_firmware.name,
            response.identify_firmware.version
        )

    def load_firmware(self, firmware: bytes, progress_cb: Optional[Callable[[float], None]]) -> None:
        """
        Load firmware to io4edge device.
        `firmware` must be the raw binary file, not a .fwpkg file.
        @param firmware: firmware binary data
        @param progress_cb: callback function that is called with progress updates (0-100)
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        chunk_number = 0
        total_size = len(firmware)
        transferred_size = 0

        while True:
            chunk_size = min(len(firmware), 1024)
            chunk = firmware[:chunk_size]
            firmware = firmware[chunk_size:]
            chunk_cmd = Pb.LoadFirmwareChunkCommand(
                chunk_number=chunk_number,
                is_last_chunk=len(firmware) == 0,
                data=chunk,
            )
            cmd = Pb.CoreCommand(
                id=Pb.CommandId.LOAD_FIRMWARE_CHUNK,
                load_firmware_chunk=chunk_cmd)
            self.command(cmd, Pb.CoreResponse())
            transferred_size += chunk_size
            if progress_cb:
                # Update progress callback with percentage
                progress_cb(
                    transferred_size / total_size * 100 if total_size > 0 else 100)
            if len(firmware) == 0:
                # Last chunk, break the loop
                break
            chunk_number += 1

    def restart(self) -> None:
        """
        Restart the io4edge device.
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        cmd = Pb.CoreCommand(id=Pb.CommandId.RESTART)
        self.command(cmd, Pb.CoreResponse())

    def set_persistent_parameter(self, name: str, value: str) -> None:
        """
        Set a persistent parameter on the io4edge device.
        @param name: name of the parameter
        @param value: value of the parameter
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        cmd = Pb.CoreCommand(
            id=Pb.CommandId.SET_PERSISTENT_PARAMETER,
            set_persistent_parameter=Pb.SetPersistentParameterCommand(
                name=name, value=value))
        self.command(cmd, Pb.CoreResponse())

    def get_persistent_parameter(self, name: str) -> str:
        """
        Get a persistent parameter from the io4edge device.
        @param name: name of the parameter
        @return: value of the parameter
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        cmd = Pb.CoreCommand(
            id=Pb.CommandId.GET_PERSISTENT_PARAMETER,
            get_persistent_parameter=Pb.GetPersistentParameterCommand(name=name))
        response = Pb.CoreResponse()
        self.command(cmd, response)
        return response.persistent_parameter.value

    def get_reset_reason(self) -> str:
        """
        Get the reason for the last reset of the io4edge device.
        @return: reset reason as a string
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        cmd = Pb.CoreCommand(id=Pb.CommandId.GET_RESET_REASON)
        response = Pb.CoreResponse()
        self.command(cmd, response)
        return response.reset_reason.reason



# SPDX-License-Identifer: Apache-2.0
import google.protobuf.any_pb2 as AnyPb

URL_PREFIX = "type.googleapis.com/"


def pb_any_unpack(from_, to):
    """
    Unpack the protobuf "any" "from_" message and place the result in the "to" message.
    """
    any = AnyPb.Any()
    any.CopyFrom(from_)
    pb_any_fix_url(any)
    any.Unpack(to)


def pb_any_fix_url(any):
    """
    Fix the type_url of a protobuf Any message.
    If it is missing the URL_PREFIX, add it, otherwise the unpacking will fail.
    io4edge devices do not send the URL_PREFIX.
    """
    if not any.type_url.startswith(URL_PREFIX):
        any.type_url = URL_PREFIX + any.type_url



class FirmwareIdentification:
    """
    Class to hold firmware identification information.
    Contains the title and version of the firmware.
    """
    def __init__(self, title: str, version: str):
        self.title = title
        self.version = version

    def __str__(self):
        return f"{self.title} v{self.version}"

class HardwareIdentification:
    """
    Class to hold hardware identification information.
    Contains the title and version of the hardware.
    """
    def __init__(self, root_article: str, major_version: str, serial_number: str):
        self.root_article = root_article
        self.major_version = major_version
        self.serial_number = serial_number

    def __str__(self):
        return f"{self.root_article} v{self.major_version} SN: {self.serial_number}"



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: core_api/v1alpha2/io4edge_core_api.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n(core_api/v1alpha2/io4edge_core_api.proto\x12\x0eio4edgeCoreApi\"U\n\x18LoadFirmwareChunkCommand\x12\x14\n\x0c\x63hunk_number\x18\x01 \x01(\r\x12\x15\n\ris_last_chunk\x18\x02 \x01(\x08\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\x0c\"}\n$ProgramHardwareIdentificationCommand\x12\x11\n\tsignature\x18\x01 \x01(\t\x12\x14\n\x0croot_article\x18\x02 \x01(\t\x12\x15\n\rmajor_version\x18\x03 \x01(\r\x12\x15\n\rserial_number\x18\x04 \x01(\t\"<\n\x1dSetPersistentParameterCommand\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"-\n\x1dGetPersistentParameterCommand\x12\x0c\n\x04name\x18\x01 \x01(\t\">\n\x19ReadPartitionChunkCommand\x12\x11\n\tpart_name\x18\x01 \x01(\t\x12\x0e\n\x06offset\x18\x02 \x01(\r\"^\n\x18IdentifyHardwareResponse\x12\x14\n\x0croot_article\x18\x01 \x01(\t\x12\x15\n\rmajor_version\x18\x02 \x01(\r\x12\x15\n\rserial_number\x18\x03 \x01(\t\"9\n\x18IdentifyFirmwareResponse\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\"/\n\x1eGetPersistentParameterResponse\x12\r\n\x05value\x18\x01 \x01(\t\"M\n\x1aReadPartitionChunkResponse\x12\x11\n\tpart_name\x18\x01 \x01(\t\x12\x0e\n\x06offset\x18\x02 \x01(\r\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\x0c\"(\n\x16GetResetReasonResponse\x12\x0e\n\x06reason\x18\x01 \x01(\t\"\xd7\x03\n\x0b\x43oreCommand\x12%\n\x02id\x18\x01 \x01(\x0e\x32\x19.io4edgeCoreApi.CommandId\x12G\n\x13load_firmware_chunk\x18\x02 \x01(\x0b\x32(.io4edgeCoreApi.LoadFirmwareChunkCommandH\x00\x12_\n\x1fprogram_hardware_identification\x18\x03 \x01(\x0b\x32\x34.io4edgeCoreApi.ProgramHardwareIdentificationCommandH\x00\x12Q\n\x18set_persistent_parameter\x18\x04 \x01(\x0b\x32-.io4edgeCoreApi.SetPersistentParameterCommandH\x00\x12Q\n\x18get_persistent_parameter\x18\x05 \x01(\x0b\x32-.io4edgeCoreApi.GetPersistentParameterCommandH\x00\x12I\n\x14read_partition_chunk\x18\x06 \x01(\x0b\x32).io4edgeCoreApi.ReadPartitionChunkCommandH\x00\x42\x06\n\x04\x64\x61ta\"\xe7\x03\n\x0c\x43oreResponse\x12%\n\x02id\x18\x01 \x01(\x0e\x32\x19.io4edgeCoreApi.CommandId\x12&\n\x06status\x18\x02 \x01(\x0e\x32\x16.io4edgeCoreApi.Status\x12\x16\n\x0erestarting_now\x18\x03 \x01(\x08\x12\x45\n\x11identify_hardware\x18\x04 \x01(\x0b\x32(.io4edgeCoreApi.IdentifyHardwareResponseH\x00\x12\x45\n\x11identify_firmware\x18\x05 \x01(\x0b\x32(.io4edgeCoreApi.IdentifyFirmwareResponseH\x00\x12N\n\x14persistent_parameter\x18\x06 \x01(\x0b\x32..io4edgeCoreApi.GetPersistentParameterResponseH\x00\x12J\n\x14read_partition_chunk\x18\x07 \x01(\x0b\x32*.io4edgeCoreApi.ReadPartitionChunkResponseH\x00\x12>\n\x0creset_reason\x18\x08 \x01(\x0b\x32&.io4edgeCoreApi.GetResetReasonResponseH\x00\x42\x06\n\x04\x64\x61ta*\xf0\x01\n\tCommandId\x12\x15\n\x11IDENTIFY_HARDWARE\x10\x00\x12\x15\n\x11IDENTIFY_FIRMWARE\x10\x01\x12\x17\n\x13LOAD_FIRMWARE_CHUNK\x10\x02\x12#\n\x1fPROGRAM_HARDWARE_IDENTIFICATION\x10\x03\x12\x0b\n\x07RESTART\x10\x04\x12\x1c\n\x18SET_PERSISTENT_PARAMETER\x10\x05\x12\x1c\n\x18GET_PERSISTENT_PARAMETER\x10\x06\x12\x18\n\x14READ_PARTITION_CHUNK\x10\x07\x12\x14\n\x10GET_RESET_REASON\x10\x08*\xd8\x01\n\x06Status\x12\x06\n\x02OK\x10\x00\x12\x13\n\x0fUNKNOWN_COMMAND\x10\x01\x12\x15\n\x11ILLEGAL_PARAMETER\x10\x02\x12\x11\n\rBAD_CHUNK_SEQ\x10\x03\x12\x12\n\x0e\x42\x41\x44_CHUNK_SIZE\x10\x04\x12\x12\n\x0eNOT_COMPATIBLE\x10\x05\x12\x12\n\x0eINTERNAL_ERROR\x10\x06\x12\x15\n\x11PROGRAMMING_ERROR\x10\x07\x12\x13\n\x0fNO_HW_INVENTORY\x10\x08\x12\x1f\n\x1bTHIS_VERSION_FAILED_ALREADY\x10\tB\x13Z\x11\x63ore_api/v1alpha2b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'core_api.v1alpha2.io4edge_core_api_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\021core_api/v1alpha2'
  _globals['_COMMANDID']._serialized_start=1737
  _globals['_COMMANDID']._serialized_end=1977
  _globals['_STATUS']._serialized_start=1980
  _globals['_STATUS']._serialized_end=2196
  _globals['_LOADFIRMWARECHUNKCOMMAND']._serialized_start=60
  _globals['_LOADFIRMWARECHUNKCOMMAND']._serialized_end=145
  _globals['_PROGRAMHARDWAREIDENTIFICATIONCOMMAND']._serialized_start=147
  _globals['_PROGRAMHARDWAREIDENTIFICATIONCOMMAND']._serialized_end=272
  _globals['_SETPERSISTENTPARAMETERCOMMAND']._serialized_start=274
  _globals['_SETPERSISTENTPARAMETERCOMMAND']._serialized_end=334
  _globals['_GETPERSISTENTPARAMETERCOMMAND']._serialized_start=336
  _globals['_GETPERSISTENTPARAMETERCOMMAND']._serialized_end=381
  _globals['_READPARTITIONCHUNKCOMMAND']._serialized_start=383
  _globals['_READPARTITIONCHUNKCOMMAND']._serialized_end=445
  _globals['_IDENTIFYHARDWARERESPONSE']._serialized_start=447
  _globals['_IDENTIFYHARDWARERESPONSE']._serialized_end=541
  _globals['_IDENTIFYFIRMWARERESPONSE']._serialized_start=543
  _globals['_IDENTIFYFIRMWARERESPONSE']._serialized_end=600
  _globals['_GETPERSISTENTPARAMETERRESPONSE']._serialized_start=602
  _globals['_GETPERSISTENTPARAMETERRESPONSE']._serialized_end=649
  _globals['_READPARTITIONCHUNKRESPONSE']._serialized_start=651
  _globals['_READPARTITIONCHUNKRESPONSE']._serialized_end=728
  _globals['_GETRESETREASONRESPONSE']._serialized_start=730
  _globals['_GETRESETREASONRESPONSE']._serialized_end=770
  _globals['_CORECOMMAND']._serialized_start=773
  _globals['_CORECOMMAND']._serialized_end=1244
  _globals['_CORERESPONSE']._serialized_start=1247
  _globals['_CORERESPONSE']._serialized_end=1734
# @@protoc_insertion_point(module_scope)



# SPDX-License-Identifer: Apache-2.0
from io4edge_client.functionblock import Client as FbClient
import io4edge_client.api.analogInTypeA.python.analogInTypeA.v1alpha1.analogInTypeA_pb2 as Pb
import io4edge_client.api.io4edge.python.functionblock.v1alpha1.io4edge_functionblock_pb2 as FbPb


class Client:
    """
    analogInTypeA functionblock client.
    @param addr: address of io4edge function block (mdns name or "ip:port" address)
    @param command_timeout: timeout for commands in seconds
    """

    def __init__(self, addr: str, command_timeout=5):
        self._fb_client = FbClient("_io4edge_analogInTypeA._tcp", addr, command_timeout)

    def upload_configuration(self, config: Pb.ConfigurationSet):
        """
        Upload the configuration to the analogInTypeA functionblock.
        @param config: configuration to upload
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        self._fb_client.upload_configuration(config)

    def download_configuration(self) -> Pb.ConfigurationGetResponse:
        """
        Download the configuration from the analogInTypeA functionblock.
        @return: actual configuration
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_response = Pb.ConfigurationGetResponse()
        self._fb_client.download_configuration(Pb.ConfigurationGet(), fs_response)
        return fs_response

    def input(self, channel: int) -> bool:
        """
        Get the state of a single channel, regardless whether its configured as input or output)
        State "true" is returned if the input level is above switching threshold, "false" otherwise.
        @param channel: channel number
        @return: state of the input
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_cmd = Pb.FunctionControlGet()
        fs_cmd.single.channel = channel
        fs_response = Pb.FunctionControlGetResponse()
        self._fb_client.function_control_get(fs_cmd, fs_response)
        return fs_response.single.state

    def value(self) -> int:
        """
        read the current analog input level.

        @return: current analog input level. range -1 .. +1 (for min/max voltage or current)
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_cmd = Pb.FunctionControlGet()
        fs_response = Pb.FunctionControlGetResponse()
        self._fb_client.function_control_get(fs_cmd, fs_response)
        return fs_response.value

    def start_stream(
        self, fb_config: FbPb.StreamControl
    ):
        """
        Start streaming of transitions.
        @param fb_config: functionblock generic configuration of the stream
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        self._fb_client.start_stream(Pb.StreamControlStart(), fb_config)

    def stop_stream(self):
        """
        Stop streaming of transitions.
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        self._fb_client.stop_stream()

    def read_stream(self, timeout=None):
        """
        Read the next message from the stream.
        @param timeout: timeout in seconds
        @return: functionblock generic stream data (deliveryTimestampUs, sequence), analogInTypeA specific stream data
        @raises TimeoutError: if no data is available within the timeout
        """
        stream_data = Pb.StreamData()
        generic_stream_data = self._fb_client.read_stream(timeout, stream_data)
        return generic_stream_data, stream_data

    def close(self):
        """
        Close the connection to the function block, terminate read thread.
        After calling this method, the object is no longer usable.
        """
        self._fb_client.close()



# SPDX-License-Identifer: Apache-2.0
from zeroconf import Zeroconf
from .socket_transport import SocketTransport


class Client:
    def __init__(self, service: str, addr: str):
        # detect if addr is a service name or an IP address
        try:
            ip, port = self._net_address_split(addr)
        except ValueError:
            # addr may be a service name
            ip, port = self._find_mdns(addr + "." + service)

        #print(f"Connecting to {ip}:{port}")
        self._transport = SocketTransport(ip, port)

    def write_msg(self, msg):
        """
        Marshall msg and write it to the server
        """
        data = msg.SerializeToString()
        self._transport.write(data)

    def read_msg(self, msg, timeout):
        """
        Wait for next message from server. Unmarshall it to msg.
        Pass msg as a protobuf message type with the expected type.
        If timeout is not None, raise TimeoutError if no message is received within timeout seconds.
        """
        data = self._transport.read(timeout)
        msg.ParseFromString(bytes(data))

    @staticmethod
    def _net_address_split(addr: str):
        # split string "ip:port" into tuple (ip, port)
        fields = addr.split(":")
        if len(fields) != 2:
            raise ValueError("invalid address")
        return fields[0], int(fields[1])

    @staticmethod
    def _split_service(service: str):
        fields = service.split(".")
        if len(fields) < 3:
            raise ValueError(
                "service address not parseable (one of these are missing: instance, service, protocol)"
            )
        service = fields[-2] + "." + fields[-1]
        instance = ".".join(fields[:-2])
        return instance, service

    @staticmethod
    def _find_mdns(service: str):
        """
        Find a service using mDNS
        :param service: service name with protocol (e.g. S101-IOU01-USB-EXT-1._io4edge-core._tcp)
        """
        zeroconf = Zeroconf()
        instance, service = Client._split_service(service)

        service += ".local."
        instance = instance + "." + service

        #print("Looking for service %s %s" % (service, instance))

        info = zeroconf.get_service_info(type_=service, name=instance)
        if info:
            rv = info.parsed_addresses()[0], info.port
        else:
            rv = None, 0
        zeroconf.close()
        return rv

    def close(self):
        self._transport.close()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: binaryIoTypeC/v1alpha1/binaryIoTypeC.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*binaryIoTypeC/v1alpha1/binaryIoTypeC.proto\x12\rbinaryIoTypeC\"`\n\rChannelConfig\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\x05\x12(\n\x04mode\x18\x02 \x01(\x0e\x32\x1a.binaryIoTypeC.ChannelMode\x12\x14\n\x0cinitialValue\x18\x03 \x01(\x08\"\xa0\x01\n\x10\x43onfigurationSet\x12\x33\n\rchannelConfig\x18\x01 \x03(\x0b\x32\x1c.binaryIoTypeC.ChannelConfig\x12\x1c\n\x14\x63hangeOutputWatchdog\x18\x02 \x01(\x08\x12\x1a\n\x12outputWatchdogMask\x18\x03 \x01(\x07\x12\x1d\n\x15outputWatchdogTimeout\x18\x04 \x01(\x07\"\x1a\n\x18\x43onfigurationSetResponse\"\x12\n\x10\x43onfigurationGet\"\x8a\x01\n\x18\x43onfigurationGetResponse\x12\x33\n\rchannelConfig\x18\x01 \x03(\x0b\x32\x1c.binaryIoTypeC.ChannelConfig\x12\x1a\n\x12outputWatchdogMask\x18\x03 \x01(\x07\x12\x1d\n\x15outputWatchdogTimeout\x18\x04 \x01(\x07\"\x17\n\x15\x43onfigurationDescribe\"9\n\x1d\x43onfigurationDescribeResponse\x12\x18\n\x10numberOfChannels\x18\x01 \x01(\x07\"\xd1\x01\n\x15\x43onfigurationResponse\x12\x36\n\x03get\x18\x01 \x01(\x0b\x32\'.binaryIoTypeC.ConfigurationGetResponseH\x00\x12\x36\n\x03set\x18\x02 \x01(\x0b\x32\'.binaryIoTypeC.ConfigurationSetResponseH\x00\x12@\n\x08\x64\x65scribe\x18\x03 \x01(\x0b\x32,.binaryIoTypeC.ConfigurationDescribeResponseH\x00\x42\x06\n\x04type\"\x1c\n\tGetSingle\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\x07\"\x08\n\x06GetAll\"n\n\x12\x46unctionControlGet\x12*\n\x06single\x18\x01 \x01(\x0b\x32\x18.binaryIoTypeC.GetSingleH\x00\x12$\n\x03\x61ll\x18\x02 \x01(\x0b\x32\x15.binaryIoTypeC.GetAllH\x00\x42\x06\n\x04type\"+\n\tSetSingle\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\x07\x12\r\n\x05state\x18\x02 \x01(\x08\"&\n\x06SetAll\x12\x0e\n\x06values\x18\x01 \x01(\x07\x12\x0c\n\x04mask\x18\x02 \x01(\x07\"n\n\x12\x46unctionControlSet\x12*\n\x06single\x18\x01 \x01(\x0b\x32\x18.binaryIoTypeC.SetSingleH\x00\x12$\n\x03\x61ll\x18\x02 \x01(\x0b\x32\x15.binaryIoTypeC.SetAllH\x00\x42\x06\n\x04type\"A\n\x11GetSingleResponse\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\x07\x12\r\n\x05state\x18\x02 \x01(\x08\x12\x0c\n\x04\x64iag\x18\x03 \x01(\x07\".\n\x0eGetAllResponse\x12\x0e\n\x06inputs\x18\x01 \x01(\x07\x12\x0c\n\x04\x64iag\x18\x02 \x03(\r\"\x86\x01\n\x1a\x46unctionControlGetResponse\x12\x32\n\x06single\x18\x01 \x01(\x0b\x32 .binaryIoTypeC.GetSingleResponseH\x00\x12,\n\x03\x61ll\x18\x02 \x01(\x0b\x32\x1d.binaryIoTypeC.GetAllResponseH\x00\x42\x06\n\x04type\"\x13\n\x11SetSingleResponse\"\x10\n\x0eSetAllResponse\"\x86\x01\n\x1a\x46unctionControlSetResponse\x12\x32\n\x06single\x18\x01 \x01(\x0b\x32 .binaryIoTypeC.SetSingleResponseH\x00\x12,\n\x03\x61ll\x18\x02 \x01(\x0b\x32\x1d.binaryIoTypeC.SetAllResponseH\x00\x42\x06\n\x04type\"/\n\x12StreamControlStart\x12\x19\n\x11\x63hannelFilterMask\x18\x01 \x01(\x07\"@\n\x06Sample\x12\x11\n\ttimestamp\x18\x01 \x01(\x06\x12\x0e\n\x06values\x18\x02 \x01(\x07\x12\x13\n\x0bvalue_valid\x18\x03 \x01(\x07\"4\n\nStreamData\x12&\n\x07samples\x18\x01 \x03(\x0b\x32\x15.binaryIoTypeC.Sample*S\n\x0b\x43hannelMode\x12 \n\x1c\x42INARYIOTYPEC_INPUT_TYPE_1_3\x10\x00\x12\"\n\x1e\x42INARYIOTYPEC_OUTPUT_PUSH_PULL\x10\x01*}\n\x0b\x43hannelDiag\x12\n\n\x06NoDiag\x10\x00\x12\x13\n\x0fNoSupplyVoltage\x10\x01\x12\x10\n\x0c\x43urrentLimit\x10\x02\x12\x0c\n\x08Overload\x10\x04\x12\x16\n\x12SupplyUndervoltage\x10\x08\x12\x15\n\x11SupplyOvervoltage\x10\x10\x42\x18Z\x16\x62inaryIoTypeC/v1alpha1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'binaryIoTypeC.v1alpha1.binaryIoTypeC_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\026binaryIoTypeC/v1alpha1'
  _globals['_CHANNELMODE']._serialized_start=1753
  _globals['_CHANNELMODE']._serialized_end=1836
  _globals['_CHANNELDIAG']._serialized_start=1838
  _globals['_CHANNELDIAG']._serialized_end=1963
  _globals['_CHANNELCONFIG']._serialized_start=61
  _globals['_CHANNELCONFIG']._serialized_end=157
  _globals['_CONFIGURATIONSET']._serialized_start=160
  _globals['_CONFIGURATIONSET']._serialized_end=320
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_start=322
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_end=348
  _globals['_CONFIGURATIONGET']._serialized_start=350
  _globals['_CONFIGURATIONGET']._serialized_end=368
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_start=371
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_end=509
  _globals['_CONFIGURATIONDESCRIBE']._serialized_start=511
  _globals['_CONFIGURATIONDESCRIBE']._serialized_end=534
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_start=536
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_end=593
  _globals['_CONFIGURATIONRESPONSE']._serialized_start=596
  _globals['_CONFIGURATIONRESPONSE']._serialized_end=805
  _globals['_GETSINGLE']._serialized_start=807
  _globals['_GETSINGLE']._serialized_end=835
  _globals['_GETALL']._serialized_start=837
  _globals['_GETALL']._serialized_end=845
  _globals['_FUNCTIONCONTROLGET']._serialized_start=847
  _globals['_FUNCTIONCONTROLGET']._serialized_end=957
  _globals['_SETSINGLE']._serialized_start=959
  _globals['_SETSINGLE']._serialized_end=1002
  _globals['_SETALL']._serialized_start=1004
  _globals['_SETALL']._serialized_end=1042
  _globals['_FUNCTIONCONTROLSET']._serialized_start=1044
  _globals['_FUNCTIONCONTROLSET']._serialized_end=1154
  _globals['_GETSINGLERESPONSE']._serialized_start=1156
  _globals['_GETSINGLERESPONSE']._serialized_end=1221
  _globals['_GETALLRESPONSE']._serialized_start=1223
  _globals['_GETALLRESPONSE']._serialized_end=1269
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_start=1272
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_end=1406
  _globals['_SETSINGLERESPONSE']._serialized_start=1408
  _globals['_SETSINGLERESPONSE']._serialized_end=1427
  _globals['_SETALLRESPONSE']._serialized_start=1429
  _globals['_SETALLRESPONSE']._serialized_end=1445
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_start=1448
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_end=1582
  _globals['_STREAMCONTROLSTART']._serialized_start=1584
  _globals['_STREAMCONTROLSTART']._serialized_end=1631
  _globals['_SAMPLE']._serialized_start=1633
  _globals['_SAMPLE']._serialized_end=1697
  _globals['_STREAMDATA']._serialized_start=1699
  _globals['_STREAMDATA']._serialized_end=1751
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: pixelDisplay/v1alpha1/pixelDisplay.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n(pixelDisplay/v1alpha1/pixelDisplay.proto\x12\x0cpixelDisplay\"\x12\n\x10\x43onfigurationSet\"\x1a\n\x18\x43onfigurationSetResponse\"\x12\n\x10\x43onfigurationGet\"\x1a\n\x18\x43onfigurationGetResponse\"\x17\n\x15\x43onfigurationDescribe\"d\n\x1d\x43onfigurationDescribeResponse\x12\x14\n\x0cheight_pixel\x18\x01 \x01(\x07\x12\x13\n\x0bwidth_pixel\x18\x02 \x01(\x07\x12\x18\n\x10max_num_of_pixel\x18\x03 \x01(\x07\"\xce\x01\n\x15\x43onfigurationResponse\x12\x35\n\x03get\x18\x01 \x01(\x0b\x32&.pixelDisplay.ConfigurationGetResponseH\x00\x12\x35\n\x03set\x18\x02 \x01(\x0b\x32&.pixelDisplay.ConfigurationSetResponseH\x00\x12?\n\x08\x64\x65scribe\x18\x03 \x01(\x0b\x32+.pixelDisplay.ConfigurationDescribeResponseH\x00\x42\x06\n\x04type\"\x14\n\x12\x46unctionControlGet\"N\n\x0cSetPixelArea\x12\x0f\n\x07start_x\x18\x01 \x01(\x07\x12\x0f\n\x07start_y\x18\x02 \x01(\x07\x12\r\n\x05\x65nd_x\x18\x03 \x01(\x07\x12\r\n\x05image\x18\x04 \x01(\x0c\"\x1a\n\x0cSetDisplayOn\x12\n\n\x02on\x18\x01 \x01(\x08\"\x84\x01\n\x12\x46unctionControlSet\x12\x34\n\x0eset_pixel_area\x18\x01 \x01(\x0b\x32\x1a.pixelDisplay.SetPixelAreaH\x00\x12\x30\n\ndisplay_on\x18\x02 \x01(\x0b\x32\x1a.pixelDisplay.SetDisplayOnH\x00\x42\x06\n\x04type\"\x1c\n\x1a\x46unctionControlGetResponse\"\x16\n\x14SetPixelAreaResponse\"\x16\n\x14SetDisplayOnResponse\"\x9c\x01\n\x1a\x46unctionControlSetResponse\x12<\n\x0eset_pixel_area\x18\x01 \x01(\x0b\x32\".pixelDisplay.SetPixelAreaResponseH\x00\x12\x38\n\ndisplay_on\x18\x02 \x01(\x0b\x32\".pixelDisplay.SetDisplayOnResponseH\x00\x42\x06\n\x04type\"\x14\n\x12StreamControlStart\"\x0c\n\nStreamDataB\x17Z\x15pixelDisplay/v1alpha1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'pixelDisplay.v1alpha1.pixelDisplay_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\025pixelDisplay/v1alpha1'
  _globals['_CONFIGURATIONSET']._serialized_start=58
  _globals['_CONFIGURATIONSET']._serialized_end=76
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_start=78
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_end=104
  _globals['_CONFIGURATIONGET']._serialized_start=106
  _globals['_CONFIGURATIONGET']._serialized_end=124
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_start=126
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_end=152
  _globals['_CONFIGURATIONDESCRIBE']._serialized_start=154
  _globals['_CONFIGURATIONDESCRIBE']._serialized_end=177
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_start=179
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_end=279
  _globals['_CONFIGURATIONRESPONSE']._serialized_start=282
  _globals['_CONFIGURATIONRESPONSE']._serialized_end=488
  _globals['_FUNCTIONCONTROLGET']._serialized_start=490
  _globals['_FUNCTIONCONTROLGET']._serialized_end=510
  _globals['_SETPIXELAREA']._serialized_start=512
  _globals['_SETPIXELAREA']._serialized_end=590
  _globals['_SETDISPLAYON']._serialized_start=592
  _globals['_SETDISPLAYON']._serialized_end=618
  _globals['_FUNCTIONCONTROLSET']._serialized_start=621
  _globals['_FUNCTIONCONTROLSET']._serialized_end=753
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_start=755
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_end=783
  _globals['_SETPIXELAREARESPONSE']._serialized_start=785
  _globals['_SETPIXELAREARESPONSE']._serialized_end=807
  _globals['_SETDISPLAYONRESPONSE']._serialized_start=809
  _globals['_SETDISPLAYONRESPONSE']._serialized_end=831
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_start=834
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_end=990
  _globals['_STREAMCONTROLSTART']._serialized_start=992
  _globals['_STREAMCONTROLSTART']._serialized_end=1012
  _globals['_STREAMDATA']._serialized_start=1014
  _globals['_STREAMDATA']._serialized_end=1026
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: binaryIoTypeB/v1alpha1/binaryIoTypeB.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*binaryIoTypeB/v1alpha1/binaryIoTypeB.proto\x12\rbinaryIoTypeB\"\x12\n\x10\x43onfigurationSet\"\x1a\n\x18\x43onfigurationSetResponse\"\x12\n\x10\x43onfigurationGet\"\x1a\n\x18\x43onfigurationGetResponse\"\x17\n\x15\x43onfigurationDescribe\"T\n\rChannelConfig\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\x07\x12\x32\n\tdirection\x18\x02 \x01(\x0e\x32\x1f.binaryIoTypeB.ChannelDirection\"T\n\x1d\x43onfigurationDescribeResponse\x12\x33\n\rchannelConfig\x18\x01 \x03(\x0b\x32\x1c.binaryIoTypeB.ChannelConfig\"\xd1\x01\n\x15\x43onfigurationResponse\x12\x36\n\x03get\x18\x01 \x01(\x0b\x32\'.binaryIoTypeB.ConfigurationGetResponseH\x00\x12\x36\n\x03set\x18\x02 \x01(\x0b\x32\'.binaryIoTypeB.ConfigurationSetResponseH\x00\x12@\n\x08\x64\x65scribe\x18\x03 \x01(\x0b\x32,.binaryIoTypeB.ConfigurationDescribeResponseH\x00\x42\x06\n\x04type\"\x1c\n\tGetSingle\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\x07\"\x16\n\x06GetAll\x12\x0c\n\x04mask\x18\x01 \x01(\x07\"n\n\x12\x46unctionControlGet\x12*\n\x06single\x18\x01 \x01(\x0b\x32\x18.binaryIoTypeB.GetSingleH\x00\x12$\n\x03\x61ll\x18\x02 \x01(\x0b\x32\x15.binaryIoTypeB.GetAllH\x00\x42\x06\n\x04type\"+\n\tSetSingle\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\x07\x12\r\n\x05state\x18\x02 \x01(\x08\"&\n\x06SetAll\x12\x0e\n\x06values\x18\x01 \x01(\x07\x12\x0c\n\x04mask\x18\x02 \x01(\x07\"n\n\x12\x46unctionControlSet\x12*\n\x06single\x18\x01 \x01(\x0b\x32\x18.binaryIoTypeB.SetSingleH\x00\x12$\n\x03\x61ll\x18\x02 \x01(\x0b\x32\x15.binaryIoTypeB.SetAllH\x00\x42\x06\n\x04type\"3\n\x11GetSingleResponse\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\x07\x12\r\n\x05state\x18\x02 \x01(\x08\" \n\x0eGetAllResponse\x12\x0e\n\x06inputs\x18\x01 \x01(\x07\"\x86\x01\n\x1a\x46unctionControlGetResponse\x12\x32\n\x06single\x18\x01 \x01(\x0b\x32 .binaryIoTypeB.GetSingleResponseH\x00\x12,\n\x03\x61ll\x18\x02 \x01(\x0b\x32\x1d.binaryIoTypeB.GetAllResponseH\x00\x42\x06\n\x04type\"\x13\n\x11SetSingleResponse\"\x10\n\x0eSetAllResponse\"\x86\x01\n\x1a\x46unctionControlSetResponse\x12\x32\n\x06single\x18\x01 \x01(\x0b\x32 .binaryIoTypeB.SetSingleResponseH\x00\x12,\n\x03\x61ll\x18\x02 \x01(\x0b\x32\x1d.binaryIoTypeB.SetAllResponseH\x00\x42\x06\n\x04type\"^\n\x10SubscribeChannel\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\x07\x12\x39\n\x10subscriptionType\x18\x02 \x01(\x0e\x32\x1f.binaryIoTypeB.SubscriptionType\"O\n\x12StreamControlStart\x12\x39\n\x10subscribeChannel\x18\x01 \x03(\x0b\x32\x1f.binaryIoTypeB.SubscribeChannel\"+\n\x06Sample\x12\x11\n\ttimestamp\x18\x01 \x01(\x06\x12\x0e\n\x06inputs\x18\x02 \x01(\x07\"4\n\nStreamData\x12&\n\x07samples\x18\x01 \x03(\x0b\x32\x15.binaryIoTypeB.Sample*e\n\x10\x43hannelDirection\x12\x17\n\x13\x42INARYIOTYPEB_INPUT\x10\x00\x12\x18\n\x14\x42INARYIOTYPEB_OUTPUT\x10\x01\x12\x1e\n\x1a\x42INARYIOTYPEB_INPUT_OUTPUT\x10\x02*v\n\x10SubscriptionType\x12 \n\x1c\x42INARYIOTYPEB_ON_RISING_EDGE\x10\x00\x12!\n\x1d\x42INARYIOTYPEB_ON_FALLING_EDGE\x10\x01\x12\x1d\n\x19\x42INARYIOTYPEB_ON_ANY_EDGE\x10\x02\x42\x18Z\x16\x62inaryIoTypeB/v1alpha1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'binaryIoTypeB.v1alpha1.binaryIoTypeB_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\026binaryIoTypeB/v1alpha1'
  _globals['_CHANNELDIRECTION']._serialized_start=1605
  _globals['_CHANNELDIRECTION']._serialized_end=1706
  _globals['_SUBSCRIPTIONTYPE']._serialized_start=1708
  _globals['_SUBSCRIPTIONTYPE']._serialized_end=1826
  _globals['_CONFIGURATIONSET']._serialized_start=61
  _globals['_CONFIGURATIONSET']._serialized_end=79
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_start=81
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_end=107
  _globals['_CONFIGURATIONGET']._serialized_start=109
  _globals['_CONFIGURATIONGET']._serialized_end=127
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_start=129
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_end=155
  _globals['_CONFIGURATIONDESCRIBE']._serialized_start=157
  _globals['_CONFIGURATIONDESCRIBE']._serialized_end=180
  _globals['_CHANNELCONFIG']._serialized_start=182
  _globals['_CHANNELCONFIG']._serialized_end=266
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_start=268
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_end=352
  _globals['_CONFIGURATIONRESPONSE']._serialized_start=355
  _globals['_CONFIGURATIONRESPONSE']._serialized_end=564
  _globals['_GETSINGLE']._serialized_start=566
  _globals['_GETSINGLE']._serialized_end=594
  _globals['_GETALL']._serialized_start=596
  _globals['_GETALL']._serialized_end=618
  _globals['_FUNCTIONCONTROLGET']._serialized_start=620
  _globals['_FUNCTIONCONTROLGET']._serialized_end=730
  _globals['_SETSINGLE']._serialized_start=732
  _globals['_SETSINGLE']._serialized_end=775
  _globals['_SETALL']._serialized_start=777
  _globals['_SETALL']._serialized_end=815
  _globals['_FUNCTIONCONTROLSET']._serialized_start=817
  _globals['_FUNCTIONCONTROLSET']._serialized_end=927
  _globals['_GETSINGLERESPONSE']._serialized_start=929
  _globals['_GETSINGLERESPONSE']._serialized_end=980
  _globals['_GETALLRESPONSE']._serialized_start=982
  _globals['_GETALLRESPONSE']._serialized_end=1014
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_start=1017
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_end=1151
  _globals['_SETSINGLERESPONSE']._serialized_start=1153
  _globals['_SETSINGLERESPONSE']._serialized_end=1172
  _globals['_SETALLRESPONSE']._serialized_start=1174
  _globals['_SETALLRESPONSE']._serialized_end=1190
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_start=1193
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_end=1327
  _globals['_SUBSCRIBECHANNEL']._serialized_start=1329
  _globals['_SUBSCRIBECHANNEL']._serialized_end=1423
  _globals['_STREAMCONTROLSTART']._serialized_start=1425
  _globals['_STREAMCONTROLSTART']._serialized_end=1504
  _globals['_SAMPLE']._serialized_start=1506
  _globals['_SAMPLE']._serialized_end=1549
  _globals['_STREAMDATA']._serialized_start=1551
  _globals['_STREAMDATA']._serialized_end=1603
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: mvbSniffer/v1/telegram.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1cmvbSniffer/v1/telegram.proto\x12\nmvbSniffer\"\xe6\x04\n\x08Telegram\x12\x11\n\ttimestamp\x18\x01 \x01(\x06\x12\r\n\x05state\x18\x02 \x01(\r\x12\'\n\x04type\x18\x03 \x01(\x0e\x32\x19.mvbSniffer.Telegram.Type\x12\x0f\n\x07\x61\x64\x64ress\x18\x04 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x05 \x01(\x0c\x12\x13\n\x0btelegram_nr\x18\x06 \x01(\x04\x12\'\n\x04line\x18\x07 \x01(\x0e\x32\x19.mvbSniffer.Telegram.Line\"S\n\x05State\x12\x0f\n\x0bkSuccessful\x10\x00\x12\r\n\tkTimedOut\x10\x01\x12\x14\n\x10kMissedMVBFrames\x10\x02\x12\x14\n\x10kMissedTelegrams\x10\x04\"\xc4\x02\n\x04Type\x12\x15\n\x11kProcessData16Bit\x10\x00\x12\x15\n\x11kProcessData32Bit\x10\x01\x12\x15\n\x11kProcessData64Bit\x10\x02\x12\x16\n\x12kProcessData128Bit\x10\x03\x12\x16\n\x12kProcessData256Bit\x10\x04\x12\x0f\n\x0bkReserved_1\x10\x05\x12\x0f\n\x0bkReserved_2\x10\x06\x12\x0f\n\x0bkReserved_3\x10\x07\x12\x17\n\x13kMastershipTransfer\x10\x08\x12\x11\n\rkGeneralEvent\x10\t\x12\x0f\n\x0bkReserved_4\x10\n\x12\x0f\n\x0bkReserved_5\x10\x0b\x12\x10\n\x0ckMessageData\x10\x0c\x12\x0f\n\x0bkGroupEvent\x10\r\x12\x10\n\x0ckSingleEvent\x10\x0e\x12\x11\n\rkDeviceStatus\x10\x0f\"\x16\n\x04Line\x12\x06\n\x02kA\x10\x00\x12\x06\n\x02kB\x10\x01\"9\n\x12TelegramCollection\x12#\n\x05\x65ntry\x18\x01 \x03(\x0b\x32\x14.mvbSniffer.TelegramB\x0fZ\rmvbSniffer/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'mvbSniffer.v1.telegram_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\rmvbSniffer/v1'
  _globals['_TELEGRAM']._serialized_start=45
  _globals['_TELEGRAM']._serialized_end=659
  _globals['_TELEGRAM_STATE']._serialized_start=225
  _globals['_TELEGRAM_STATE']._serialized_end=308
  _globals['_TELEGRAM_TYPE']._serialized_start=311
  _globals['_TELEGRAM_TYPE']._serialized_end=635
  _globals['_TELEGRAM_LINE']._serialized_start=637
  _globals['_TELEGRAM_LINE']._serialized_end=659
  _globals['_TELEGRAMCOLLECTION']._serialized_start=661
  _globals['_TELEGRAMCOLLECTION']._serialized_end=718
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: mvbSniffer/v1/mvbSniffer.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1emvbSniffer/v1/mvbSniffer.proto\x12\nmvbSniffer\"\x12\n\x10\x43onfigurationSet\"\x1a\n\x18\x43onfigurationSetResponse\"\x12\n\x10\x43onfigurationGet\"\x1a\n\x18\x43onfigurationGetResponse\"\x17\n\x15\x43onfigurationDescribe\"\x1f\n\x1d\x43onfigurationDescribeResponse\"\x14\n\x12\x46unctionControlGet\"/\n\x12\x46unctionControlSet\x12\x19\n\x11generator_pattern\x18\x01 \x01(\t\"\x1c\n\x1a\x46unctionControlGetResponse\"\x1c\n\x1a\x46unctionControlSetResponse\"a\n\nFilterMask\x12\x13\n\x0b\x66_code_mask\x18\x01 \x01(\r\x12\x0f\n\x07\x61\x64\x64ress\x18\x02 \x01(\r\x12\x0c\n\x04mask\x18\x03 \x01(\r\x12\x1f\n\x17include_timedout_frames\x18\x04 \x01(\x08\"<\n\x12StreamControlStart\x12&\n\x06\x66ilter\x18\x01 \x03(\x0b\x32\x16.mvbSniffer.FilterMaskB\x0fZ\rmvbSniffer/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'mvbSniffer.v1.mvbSniffer_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\rmvbSniffer/v1'
  _globals['_CONFIGURATIONSET']._serialized_start=46
  _globals['_CONFIGURATIONSET']._serialized_end=64
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_start=66
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_end=92
  _globals['_CONFIGURATIONGET']._serialized_start=94
  _globals['_CONFIGURATIONGET']._serialized_end=112
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_start=114
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_end=140
  _globals['_CONFIGURATIONDESCRIBE']._serialized_start=142
  _globals['_CONFIGURATIONDESCRIBE']._serialized_end=165
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_start=167
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_end=198
  _globals['_FUNCTIONCONTROLGET']._serialized_start=200
  _globals['_FUNCTIONCONTROLGET']._serialized_end=220
  _globals['_FUNCTIONCONTROLSET']._serialized_start=222
  _globals['_FUNCTIONCONTROLSET']._serialized_end=269
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_start=271
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_end=299
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_start=301
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_end=329
  _globals['_FILTERMASK']._serialized_start=331
  _globals['_FILTERMASK']._serialized_end=428
  _globals['_STREAMCONTROLSTART']._serialized_start=430
  _globals['_STREAMCONTROLSTART']._serialized_end=490
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: mvbDebug/v1/mvbDebug.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1amvbDebug/v1/mvbDebug.proto\x12\x08mvbDebug\"\x12\n\x10\x43onfigurationSet\"\x1a\n\x18\x43onfigurationSetResponse\"\x12\n\x10\x43onfigurationGet\"\x1a\n\x18\x43onfigurationGetResponse\"\x17\n\x15\x43onfigurationDescribe\"\x1f\n\x1d\x43onfigurationDescribeResponse\"\x14\n\x12\x46unctionControlGet\"/\n\x12\x46unctionControlSet\x12\x19\n\x11generator_pattern\x18\x01 \x01(\t\"+\n\x1a\x46unctionControlGetResponse\x12\r\n\x05value\x18\x01 \x01(\x02\"\x1c\n\x1a\x46unctionControlSetResponse\"\x14\n\x12StreamControlStart\"#\n\x06Sample\x12\x19\n\x11transitions_block\x18\x01 \x01(\x0c\"/\n\nStreamData\x12!\n\x07samples\x18\x01 \x03(\x0b\x32\x10.mvbDebug.SampleB\rZ\x0bmvbDebug/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'mvbDebug.v1.mvbDebug_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\013mvbDebug/v1'
  _globals['_CONFIGURATIONSET']._serialized_start=40
  _globals['_CONFIGURATIONSET']._serialized_end=58
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_start=60
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_end=86
  _globals['_CONFIGURATIONGET']._serialized_start=88
  _globals['_CONFIGURATIONGET']._serialized_end=106
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_start=108
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_end=134
  _globals['_CONFIGURATIONDESCRIBE']._serialized_start=136
  _globals['_CONFIGURATIONDESCRIBE']._serialized_end=159
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_start=161
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_end=192
  _globals['_FUNCTIONCONTROLGET']._serialized_start=194
  _globals['_FUNCTIONCONTROLGET']._serialized_end=214
  _globals['_FUNCTIONCONTROLSET']._serialized_start=216
  _globals['_FUNCTIONCONTROLSET']._serialized_end=263
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_start=265
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_end=308
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_start=310
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_end=338
  _globals['_STREAMCONTROLSTART']._serialized_start=340
  _globals['_STREAMCONTROLSTART']._serialized_end=360
  _globals['_SAMPLE']._serialized_start=362
  _globals['_SAMPLE']._serialized_end=397
  _globals['_STREAMDATA']._serialized_start=399
  _globals['_STREAMDATA']._serialized_end=446
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: inventory/v1alpha1/io4edge_inventory.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*inventory/v1alpha1/io4edge_inventory.proto\x12\x10io4edgeInventory\"C\n\x04Unit\x12\x14\n\x0croot_article\x18\x01 \x01(\t\x12\x15\n\rmajor_version\x18\x02 \x01(\r\x12\x0e\n\x06serial\x18\x03 \x01(\tB\x14Z\x12inventory/v1alpha1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'inventory.v1alpha1.io4edge_inventory_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\022inventory/v1alpha1'
  _globals['_UNIT']._serialized_start=64
  _globals['_UNIT']._serialized_end=131
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: binaryIoTypeD/v1/binaryIoTypeD.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$binaryIoTypeD/v1/binaryIoTypeD.proto\x12\rbinaryIoTypeD\"\xb6\x01\n\rChannelConfig\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\x05\x12(\n\x04mode\x18\x02 \x01(\x0e\x32\x1a.binaryIoTypeD.ChannelMode\x12\x14\n\x0cinitialValue\x18\x03 \x01(\x08\x12!\n\x19overloadRecoveryTimeoutMs\x18\x04 \x01(\x05\x12\x19\n\x11watchdogTimeoutMs\x18\x05 \x01(\x05\x12\x16\n\x0e\x66rittingEnable\x18\x06 \x01(\x08\"G\n\x10\x43onfigurationSet\x12\x33\n\rchannelConfig\x18\x01 \x03(\x0b\x32\x1c.binaryIoTypeD.ChannelConfig\"\x1a\n\x18\x43onfigurationSetResponse\"\x12\n\x10\x43onfigurationGet\"O\n\x18\x43onfigurationGetResponse\x12\x33\n\rchannelConfig\x18\x01 \x03(\x0b\x32\x1c.binaryIoTypeD.ChannelConfig\"\x17\n\x15\x43onfigurationDescribe\"9\n\x1d\x43onfigurationDescribeResponse\x12\x18\n\x10numberOfChannels\x18\x01 \x01(\x07\"\x14\n\x12\x46unctionControlGet\"+\n\tSetSingle\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\x07\x12\r\n\x05state\x18\x02 \x01(\x08\"&\n\x06SetAll\x12\x0e\n\x06values\x18\x01 \x01(\x07\x12\x0c\n\x04mask\x18\x02 \x01(\x07\"n\n\x12\x46unctionControlSet\x12*\n\x06single\x18\x01 \x01(\x0b\x32\x18.binaryIoTypeD.SetSingleH\x00\x12$\n\x03\x61ll\x18\x02 \x01(\x0b\x32\x15.binaryIoTypeD.SetAllH\x00\x42\x06\n\x04type\":\n\x1a\x46unctionControlGetResponse\x12\x0e\n\x06inputs\x18\x01 \x01(\x07\x12\x0c\n\x04\x64iag\x18\x02 \x03(\r\"\x10\n\x0eSetAllResponse\"\x13\n\x11SetSingleResponse\"\x86\x01\n\x1a\x46unctionControlSetResponse\x12\x32\n\x06single\x18\x01 \x01(\x0b\x32 .binaryIoTypeD.SetSingleResponseH\x00\x12,\n\x03\x61ll\x18\x02 \x01(\x0b\x32\x1d.binaryIoTypeD.SetAllResponseH\x00\x42\x06\n\x04type\"\x14\n\x12StreamControlStart\"\x08\n\x06Sample\"\x0c\n\nStreamData*\xa1\x01\n\x0b\x43hannelMode\x12\"\n\x1e\x42INARYIOTYPED_INPUT_LOW_ACTIVE\x10\x00\x12#\n\x1f\x42INARYIOTYPED_INPUT_HIGH_ACTIVE\x10\x01\x12#\n\x1f\x42INARYIOTYPED_OUTPUT_LOW_ACTIVE\x10\x02\x12$\n BINARYIOTYPED_OUTPUT_HIGH_ACTIVE\x10\x03*<\n\x0b\x43hannelDiag\x12\n\n\x06NoDiag\x10\x00\x12\x13\n\x0fNoSupplyVoltage\x10\x01\x12\x0c\n\x08Overload\x10\x02\x42\x12Z\x10\x62inaryIoTypeD/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'binaryIoTypeD.v1.binaryIoTypeD_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\020binaryIoTypeD/v1'
  _globals['_CHANNELMODE']._serialized_start=1028
  _globals['_CHANNELMODE']._serialized_end=1189
  _globals['_CHANNELDIAG']._serialized_start=1191
  _globals['_CHANNELDIAG']._serialized_end=1251
  _globals['_CHANNELCONFIG']._serialized_start=56
  _globals['_CHANNELCONFIG']._serialized_end=238
  _globals['_CONFIGURATIONSET']._serialized_start=240
  _globals['_CONFIGURATIONSET']._serialized_end=311
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_start=313
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_end=339
  _globals['_CONFIGURATIONGET']._serialized_start=341
  _globals['_CONFIGURATIONGET']._serialized_end=359
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_start=361
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_end=440
  _globals['_CONFIGURATIONDESCRIBE']._serialized_start=442
  _globals['_CONFIGURATIONDESCRIBE']._serialized_end=465
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_start=467
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_end=524
  _globals['_FUNCTIONCONTROLGET']._serialized_start=526
  _globals['_FUNCTIONCONTROLGET']._serialized_end=546
  _globals['_SETSINGLE']._serialized_start=548
  _globals['_SETSINGLE']._serialized_end=591
  _globals['_SETALL']._serialized_start=593
  _globals['_SETALL']._serialized_end=631
  _globals['_FUNCTIONCONTROLSET']._serialized_start=633
  _globals['_FUNCTIONCONTROLSET']._serialized_end=743
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_start=745
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_end=803
  _globals['_SETALLRESPONSE']._serialized_start=805
  _globals['_SETALLRESPONSE']._serialized_end=821
  _globals['_SETSINGLERESPONSE']._serialized_start=823
  _globals['_SETSINGLERESPONSE']._serialized_end=842
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_start=845
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_end=979
  _globals['_STREAMCONTROLSTART']._serialized_start=981
  _globals['_STREAMCONTROLSTART']._serialized_end=1001
  _globals['_SAMPLE']._serialized_start=1003
  _globals['_SAMPLE']._serialized_end=1011
  _globals['_STREAMDATA']._serialized_start=1013
  _globals['_STREAMDATA']._serialized_end=1025
# @@protoc_insertion_point(module_scope)



# SPDX-License-Identifer: Apache-2.0
import unittest
import io4edge_client.base as base


class TestBase(unittest.TestCase):
    def test_net_address_split_ok(self):
        ip, port = base.Client._net_address_split("192.168.200.1:9999")
        self.assertEqual(ip, "192.168.200.1")
        self.assertEqual(port, 9999)

    def test_net_address_split_nok(self):
        with self.assertRaises(ValueError):
            ip, port = base.Client._net_address_split("abc")

    def test_split_service_ok(self):
        instance, service = base.Client._split_service(
            "iou04-usb-ext-4._io4edge-core._tcp"
        )
        self.assertEqual(instance, "iou04-usb-ext-4")
        self.assertEqual(service, "_io4edge-core._tcp")

        instance, service = base.Client._split_service("foo.bar.baz._io4edge-core._tcp")
        self.assertEqual(instance, "foo.bar.baz")
        self.assertEqual(service, "_io4edge-core._tcp")

    def test_split_service_nok(self):
        with self.assertRaises(ValueError):
            _, _ = base.Client._net_address_split("_io4edge-core._tcp")


if __name__ == "__main__":
    unittest.main()



# SPDX-License-Identifer: Apache-2.0
import socket
import struct
import select


class SocketTransport:
    def __init__(self, host, port):
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
        s.connect((host, port))
        self._socket = s

    def write(self, data: bytes) -> int:
        """
        Send the data as an io4edge message to the server
        """
        hdr = struct.pack("<HL", 0xEDFE, len(data))
        return self._socket.sendall(hdr + data)

    def read(self, timeout) -> bytes:
        """
        Wait for next io4edge message from server.
        Return payload.
        If timeout is not None, raise TimeoutError if no message is received within timeout seconds.
        """
        if timeout is not None:
            ready = select.select([self._socket], [], [], timeout)
            if not ready[0]:
                raise TimeoutError("timeout")

        hdr = self._rcv_all(6)
        if hdr[0:2] == b"\xFE\xED":
            len = struct.unpack("<L", hdr[2:6])[0]
            data = self._rcv_all(len)
            return data
        raise RuntimeError("bad magic")

    def _rcv_all(self, data_len: int) -> bytes:
        remaining = data_len
        buf = bytearray()
        while remaining > 0:
            data = self._socket.recv(remaining)
            buf.extend(data)
            remaining -= len(data)
        return buf

    def close(self):
        self._socket.close()



# SPDX-License-Identifer: Apache-2.0
from io4edge_client.functionblock import Client as FbClient
import io4edge_client.api.binaryIoTypeA.python.binaryIoTypeA.v1alpha1.binaryIoTypeA_pb2 as Pb
import io4edge_client.api.io4edge.python.functionblock.v1alpha1.io4edge_functionblock_pb2 as FbPb


class Client:
    """
    binaryIoTypeA functionblock client.
    @param addr: address of io4edge function block (mdns name or "ip:port" address)
    @param command_timeout: timeout for commands in seconds
    """

    def __init__(self, addr: str, command_timeout=5):
        self._fb_client = FbClient("_io4edge_binaryIoTypeA._tcp", addr, command_timeout)

    def upload_configuration(self, config: Pb.ConfigurationSet):
        """
        Upload the configuration to the binaryIoTypeA functionblock.
        @param config: configuration to upload
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        self._fb_client.upload_configuration(config)

    def download_configuration(self) -> Pb.ConfigurationGetResponse:
        """
        Download the configuration from the binaryIoTypeA functionblock.
        @return: actual configuration
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_response = Pb.ConfigurationGetResponse()
        self._fb_client.download_configuration(Pb.ConfigurationGet(), fs_response)
        return fs_response

    def describe(self) -> Pb.ConfigurationDescribeResponse:
        """
        Get the description from the binaryIoTypeA functionblock.
        @return: description from the binaryIoTypeA functionblock
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_response = Pb.ConfigurationDescribeResponse()
        self._fb_client.describe(Pb.ConfigurationDescribe(), fs_response)
        return fs_response

    def set_output(self, channel: int, state: bool):
        """
        Set the state of a single output.
        @param channel: channel number
        @param state: state to set. a "true" state turns on the outputs switch, a "false" state turns it off.
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_cmd = Pb.FunctionControlSet()
        fs_cmd.single.channel = channel
        fs_cmd.single.state = state
        self._fb_client.function_control_set(fs_cmd, Pb.FunctionControlSetResponse())

    def set_all_outputs(self, states: int, mask: int):
        """
        Set the state of all or a group of output channels.
        @param states: binary coded map of outputs. 0 means switch off, 1 means switch on, LSB is Channel0
        @param mask: binary coded map of outputs to be set. 0 means do not change, 1 means change, LSB is Channel0
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_cmd = Pb.FunctionControlSet()
        fs_cmd.all.values = states
        fs_cmd.all.mask = mask
        self._fb_client.function_control_set(fs_cmd, Pb.FunctionControlSetResponse())

    def exit_error_state(self):
        """
        Try to recover the binary output controller from error state.
        The binary output controller enters error state when there is an overurrent condition for a long time.
        In the error state, no outputs can be set; inputs can still be read.
        This call tells the binary output controller to try again.
        This call does however not wait if the recovery was successful or not.
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_cmd = Pb.FunctionControlSet()
        fs_cmd.exit_error.CopyFrom(Pb.SetExitError())
        self._fb_client.function_control_set(fs_cmd, Pb.FunctionControlSetResponse())

    def input(self, channel: int) -> bool:
        """
        Get the state of a single channel, regardless whether its configured as input or output)
        State "true" is returned if the input level is above switching threshold, "false" otherwise.
        @param channel: channel number
        @return: state of the input
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_cmd = Pb.FunctionControlGet()
        fs_cmd.single.channel = channel
        fs_response = Pb.FunctionControlGetResponse()
        self._fb_client.function_control_get(fs_cmd, fs_response)
        return fs_response.single.state

    def all_inputs(self, mask: int) -> int:
        """
        Get the state of all channels, regardless whether they are configured as input or output.
        Each bit in the returned state corresponds to one channel, bit0 being channel 0.
        The bit is "true" if the input level is above switching threshold, "false" otherwise.

        @param: mask to define which channels are read. 0 means mask, 1 means unmask, LSB is Channel0
        @return: state of all inputs
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_cmd = Pb.FunctionControlGet()
        fs_cmd.all.mask = mask
        fs_response = Pb.FunctionControlGetResponse()
        self._fb_client.function_control_get(fs_cmd, fs_response)
        return fs_response.all.inputs

    def start_stream(
        self, config: Pb.StreamControlStart, fb_config: FbPb.StreamControl
    ):
        """
        Start streaming of transitions.
        @param config: filter configuration of the stream
        @param fb_config: functionblock generic configuration of the stream
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        self._fb_client.start_stream(config, fb_config)

    def stop_stream(self):
        """
        Stop streaming of transitions.
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        self._fb_client.stop_stream()

    def read_stream(self, timeout=None):
        """
        Read the next message from the stream.
        @param timeout: timeout in seconds
        @return: functionblock generic stream data (deliveryTimestampUs, sequence), binaryIoTypeA specific stream data
        @raises TimeoutError: if no data is available within the timeout
        """
        stream_data = Pb.StreamData()
        generic_stream_data = self._fb_client.read_stream(timeout, stream_data)
        return generic_stream_data, stream_data

    def close(self):
        """
        Close the connection to the function block, terminate read thread.
        After calling this method, the object is no longer usable.
        """
        self._fb_client.close()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: templateInterface/v1alpha1/templateInterface.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n2templateInterface/v1alpha1/templateInterface.proto\x12\x11templateInterface\"\'\n\x10\x43onfigurationSet\x12\x13\n\x0bsample_rate\x18\x01 \x01(\x07\"\x1a\n\x18\x43onfigurationSetResponse\"\x12\n\x10\x43onfigurationGet\"/\n\x18\x43onfigurationGetResponse\x12\x13\n\x0bsample_rate\x18\x01 \x01(\x07\"\x17\n\x15\x43onfigurationDescribe\".\n\x1d\x43onfigurationDescribeResponse\x12\r\n\x05ident\x18\x01 \x01(\t\"\xdd\x01\n\x15\x43onfigurationResponse\x12:\n\x03get\x18\x01 \x01(\x0b\x32+.templateInterface.ConfigurationGetResponseH\x00\x12:\n\x03set\x18\x02 \x01(\x0b\x32+.templateInterface.ConfigurationSetResponseH\x00\x12\x44\n\x08\x64\x65scribe\x18\x03 \x01(\x0b\x32\x30.templateInterface.ConfigurationDescribeResponseH\x00\x42\x06\n\x04type\"\x14\n\x12\x46unctionControlGet\"#\n\x12\x46unctionControlSet\x12\r\n\x05value\x18\x01 \x01(\x07\"+\n\x1a\x46unctionControlGetResponse\x12\r\n\x05value\x18\x01 \x01(\x07\"\x1c\n\x1a\x46unctionControlSetResponse\"$\n\x12StreamControlStart\x12\x0e\n\x06modulo\x18\x01 \x01(\x07\"*\n\x06Sample\x12\x11\n\ttimestamp\x18\x01 \x01(\x06\x12\r\n\x05value\x18\x02 \x01(\x07\"8\n\nStreamData\x12*\n\x07samples\x18\x01 \x03(\x0b\x32\x19.templateInterface.SampleB\x1cZ\x1atemplateInterface/v1alpha1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'templateInterface.v1alpha1.templateInterface_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\032templateInterface/v1alpha1'
  _globals['_CONFIGURATIONSET']._serialized_start=73
  _globals['_CONFIGURATIONSET']._serialized_end=112
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_start=114
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_end=140
  _globals['_CONFIGURATIONGET']._serialized_start=142
  _globals['_CONFIGURATIONGET']._serialized_end=160
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_start=162
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_end=209
  _globals['_CONFIGURATIONDESCRIBE']._serialized_start=211
  _globals['_CONFIGURATIONDESCRIBE']._serialized_end=234
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_start=236
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_end=282
  _globals['_CONFIGURATIONRESPONSE']._serialized_start=285
  _globals['_CONFIGURATIONRESPONSE']._serialized_end=506
  _globals['_FUNCTIONCONTROLGET']._serialized_start=508
  _globals['_FUNCTIONCONTROLGET']._serialized_end=528
  _globals['_FUNCTIONCONTROLSET']._serialized_start=530
  _globals['_FUNCTIONCONTROLSET']._serialized_end=565
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_start=567
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_end=610
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_start=612
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_end=640
  _globals['_STREAMCONTROLSTART']._serialized_start=642
  _globals['_STREAMCONTROLSTART']._serialized_end=678
  _globals['_SAMPLE']._serialized_start=680
  _globals['_SAMPLE']._serialized_end=722
  _globals['_STREAMDATA']._serialized_start=724
  _globals['_STREAMDATA']._serialized_end=780
# @@protoc_insertion_point(module_scope)



# SPDX-License-Identifer: Apache-2.0
from io4edge_client.functionblock import Client as FbClient
import io4edge_client.api.binaryIoTypeB.python.binaryIoTypeB.v1alpha1.binaryIoTypeB_pb2 as Pb
import io4edge_client.api.io4edge.python.functionblock.v1alpha1.io4edge_functionblock_pb2 as FbPb


class Client:
    """
    binaryIoTypeB functionblock client.
    @param addr: address of io4edge function block (mdns name or "ip:port" address)
    @param command_timeout: timeout for commands in seconds
    """

    def __init__(self, addr: str, command_timeout=5):
        self._fb_client = FbClient("_io4edge_binaryIoTypeB._tcp", addr, command_timeout)

    def describe(self) -> Pb.ConfigurationDescribeResponse:
        """
        Get the description from the binaryIoTypeB functionblock.
        @return: description from the binaryIoTypeB functionblock
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_response = Pb.ConfigurationDescribeResponse()
        self._fb_client.describe(Pb.ConfigurationDescribe(), fs_response)
        return fs_response

    def set_output(self, channel: int, state: bool):
        """
        Set the state of a single output.
        @param channel: channel number
        @param state: state to set. a "true" state turns on the outputs switch, a "false" state turns it off.
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_cmd = Pb.FunctionControlSet()
        fs_cmd.single.channel = channel
        fs_cmd.single.state = state
        self._fb_client.function_control_set(fs_cmd, Pb.FunctionControlSetResponse())

    def set_all_outputs(self, states: int, mask: int):
        """
        Set the state of all or a group of output channels.
        @param states: binary coded map of outputs. 0 means switch off, 1 means switch on, LSB is Channel0
        @param mask: binary coded map of outputs to be set. 0 means do not change, 1 means change, LSB is Channel0
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_cmd = Pb.FunctionControlSet()
        fs_cmd.all.values = states
        fs_cmd.all.mask = mask
        self._fb_client.function_control_set(fs_cmd, Pb.FunctionControlSetResponse())

    def get_input(self, channel: int) -> bool:
        """
        Get the state of a single input.
        @param channel: channel number
        @return: state of the input
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_cmd = Pb.FunctionControlGet()
        fs_cmd.single.channel = channel
        fs_response = Pb.FunctionControlGetResponse()
        self._fb_client.function_control_get(fs_cmd, fs_response)
        return fs_response.single.state
    
    def get_all_inputs(self) -> int:
        """
        Get the state of all inputs.
        @return: binary coded map of inputs. 0 means switch off, 1 means switch on, LSB is Channel0
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_cmd = Pb.FunctionControlGet()
        fs_response = Pb.FunctionControlGetResponse()
        self._fb_client.function_control_get(fs_cmd, fs_response)
        return fs_response.all.values

    def close(self):
        """
        Close the connection to the function block, terminate read thread.
        After calling this method, the object is no longer usable.
        """
        self._fb_client.close()

    def start_stream(
        self, config: Pb.StreamControlStart, fb_config: FbPb.StreamControl
    ):
        """
        Start streaming of transitions.
        @param config: filter configuration of the stream
        @param fb_config: functionblock generic configuration of the stream
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        self._fb_client.start_stream(config, fb_config)

    def stop_stream(self):
        """
        Stop streaming of transitions.
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        self._fb_client.stop_stream()

    def read_stream(self, timeout=None):
        """
        Read the next message from the stream.
        @param timeout: timeout in seconds
        @return: functionblock generic stream data (deliveryTimestampUs, sequence), binaryIoTypeC specific stream data
        @raises TimeoutError: if no data is available within the timeout
        """
        stream_data = Pb.StreamData()
        generic_stream_data = self._fb_client.read_stream(timeout, stream_data)
        return generic_stream_data, stream_data



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: motionSensor/v1/motionSensor.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"motionSensor/v1/motionSensor.proto\x12\x0cmotionSensor\"\x80\x01\n\x10\x43onfigurationSet\x12\x1b\n\x13sample_rate_milliHz\x18\x01 \x01(\x07\x12\x14\n\x0c\x66ull_scale_g\x18\x02 \x01(\x05\x12\x1f\n\x17high_pass_filter_enable\x18\x03 \x01(\x08\x12\x18\n\x10\x62\x61nd_width_ratio\x18\x04 \x01(\x05\"\x1a\n\x18\x43onfigurationSetResponse\"\x12\n\x10\x43onfigurationGet\"\x88\x01\n\x18\x43onfigurationGetResponse\x12\x1b\n\x13sample_rate_millihz\x18\x01 \x01(\x07\x12\x14\n\x0c\x66ull_scale_g\x18\x02 \x01(\x05\x12\x1f\n\x17high_pass_filter_enable\x18\x03 \x01(\x08\x12\x18\n\x10\x62\x61nd_width_ratio\x18\x04 \x01(\x05\"\x17\n\x15\x43onfigurationDescribe\"\x1f\n\x1d\x43onfigurationDescribeResponse\"\x14\n\x12\x46unctionControlGet\"\x14\n\x12\x46unctionControlSet\"\x1c\n\x1a\x46unctionControlGetResponse\"\x1c\n\x1a\x46unctionControlSetResponse\"\x14\n\x12StreamControlStart\"<\n\x06Sample\x12\x11\n\ttimestamp\x18\x01 \x01(\x06\x12\t\n\x01x\x18\x02 \x01(\x02\x12\t\n\x01y\x18\x03 \x01(\x02\x12\t\n\x01z\x18\x04 \x01(\x02\"3\n\nStreamData\x12%\n\x07samples\x18\x01 \x03(\x0b\x32\x14.motionSensor.SampleB\x11Z\x0fmotionSensor/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'motionSensor.v1.motionSensor_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\017motionSensor/v1'
  _globals['_CONFIGURATIONSET']._serialized_start=53
  _globals['_CONFIGURATIONSET']._serialized_end=181
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_start=183
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_end=209
  _globals['_CONFIGURATIONGET']._serialized_start=211
  _globals['_CONFIGURATIONGET']._serialized_end=229
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_start=232
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_end=368
  _globals['_CONFIGURATIONDESCRIBE']._serialized_start=370
  _globals['_CONFIGURATIONDESCRIBE']._serialized_end=393
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_start=395
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_end=426
  _globals['_FUNCTIONCONTROLGET']._serialized_start=428
  _globals['_FUNCTIONCONTROLGET']._serialized_end=448
  _globals['_FUNCTIONCONTROLSET']._serialized_start=450
  _globals['_FUNCTIONCONTROLSET']._serialized_end=470
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_start=472
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_end=500
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_start=502
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_end=530
  _globals['_STREAMCONTROLSTART']._serialized_start=532
  _globals['_STREAMCONTROLSTART']._serialized_end=552
  _globals['_SAMPLE']._serialized_start=554
  _globals['_SAMPLE']._serialized_end=614
  _globals['_STREAMDATA']._serialized_start=616
  _globals['_STREAMDATA']._serialized_end=667
# @@protoc_insertion_point(module_scope)



from .protobufcom import PbCoreClient




def new_core_client(addr: str, command_timeout=5) -> PbCoreClient:
    """
    Create a new io4edge core client using protobuf communication.
    @param addr: address of io4edge function block (mdns name or "ip:port" address)
    @param command_timeout: timeout for commands in seconds
    @return: instance of PbCoreClient
    """
    # prepared to return later either a PbCoreClient or HTTPS REST API client
    return PbCoreClient(addr, command_timeout)



# SPDX-License-Identifer: Apache-2.0
from io4edge_client.functionblock import Client as FbClient
import io4edge_client.api.pixelDisplay.python.pixelDisplay.v1alpha1.pixelDisplay_pb2 as Pb
import io4edge_client.api.io4edge.python.functionblock.v1alpha1.io4edge_functionblock_pb2 as FbPb
import time
import zlib
import sys


class Client:
    """
    pixelDisplay functionblock client.
    @param addr: address of io4edge function block (mdns name or "ip:port" address)
    @param command_timeout: timeout for commands in seconds
    """

    def __init__(self, addr: str, command_timeout=5):
        self._fb_client = FbClient("_io4edge_pixelDisplay._tcp", addr, command_timeout)

    def describe(self) -> Pb.ConfigurationDescribeResponse:
        """
        Get the description from the pixelDisplay functionblock.
        @return: description from the pixelDisplay functionblock
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_response = Pb.ConfigurationDescribeResponse()
        self._fb_client.describe(Pb.ConfigurationDescribe(), fs_response)
        return fs_response

    def set_pixel_area(self, startx: int, starty: int, endx: int, pixel_area: list):
        """
        Set the pixel area of the display.
        @param startx: starting x-coordinate of the pixel area
        @param starty: starting y-coordinate of the pixel area
        @param endx: ending x-coordinate of the pixel area
        @param pixel_area: list of RGB tuples representing the pixel colors
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_cmd = Pb.FunctionControlSet()
        pixel = bytearray(len(pixel_area)*3)
        for i in range(len(pixel_area)):
            pixel[i*3+0] = pixel_area[i][0]
            pixel[i*3+1] = pixel_area[i][1]
            pixel[i*3+2] = pixel_area[i][2]
        fs_cmd.set_pixel_area.start_x = startx
        fs_cmd.set_pixel_area.start_y = starty
        fs_cmd.set_pixel_area.end_x = endx
        # compress pixel before sending to the function block
        fs_cmd.set_pixel_area.image = zlib.compress(pixel)
        self._fb_client.function_control_set(fs_cmd, Pb.FunctionControlSetResponse())

    def set_display_off(self):
        """
        Set the state of all or a group of output channels.
        @param states: binary coded map of outputs. 0 means switch off, 1 means switch on, LSB is Channel0
        @param mask: binary coded map of outputs to be set. 0 means do not change, 1 means change, LSB is Channel0
        @raises RuntimeError: if the command fails
        @raises TimeoutError: if the command times out
        """
        fs_cmd = Pb.FunctionControlSet()
        fs_cmd.display_on.on = False
        self._fb_client.function_control_set(fs_cmd, Pb.FunctionControlSetResponse())

    def close(self):
        """
        Close the connection to the function block, terminate read thread.
        After calling this method, the object is no longer usable.
        """
        self._fb_client.close()



# SPDX-License-Identifer: Apache-2.0
from setuptools import setup, find_packages
import os
import re


def read(fname):
    try:
        return open(os.path.join(os.path.dirname(__file__), fname)).read()
    except IOError:
        return "File '%s' not found.\n" % fname


def readVersion():
    txt = read("io4edge_client/version.py")
    ver = re.findall(r"([0-9]+), ([0-9]+), ([0-9]+)", txt)
    ver = ver[0]
    print("ver=%s" % ver.__str__())
    return ver[0] + "." + ver[1] + "." + ver[2]


def find_all_pbs():
    all_pbs = []
    for root, dirs, files in os.walk("io4edge_client"):
        for file in files:
            if file.endswith("_pb2.py"):
                path = os.path.join(root, file)
                path = os.sep.join(path.split(os.sep)[1:])
                all_pbs.append(path)
    return all_pbs


setup(
    name="io4edge_client",
    install_requires=[
        "protobuf",
        "zeroconf",
    ],
    version=readVersion(),
    description="A python library for io4edge devices",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    url="https://github.com/ci4rail/io4edge-client-python",
    project_urls={
        "Source Code": "https://github.com/ci4rail/io4edge-client-python.git",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    author="Ci4Rail GmbH",
    author_email="engineering@ci4rail.com",
    license_files="LICENSE",
    packages=find_packages(where="."),
    package_data={"io4edge_client": find_all_pbs()},
    setup_requires=["wheel"],
)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: binaryIoTypeA/v1alpha1/binaryIoTypeA.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*binaryIoTypeA/v1alpha1/binaryIoTypeA.proto\x12\rbinaryIoTypeA\"i\n\x10\x43onfigurationSet\x12\x1a\n\x12outputFrittingMask\x18\x01 \x01(\x07\x12\x1a\n\x12outputWatchdogMask\x18\x02 \x01(\x07\x12\x1d\n\x15outputWatchdogTimeout\x18\x03 \x01(\x07\"\x1a\n\x18\x43onfigurationSetResponse\"\x12\n\x10\x43onfigurationGet\"q\n\x18\x43onfigurationGetResponse\x12\x1a\n\x12outputFrittingMask\x18\x01 \x01(\x07\x12\x1a\n\x12outputWatchdogMask\x18\x02 \x01(\x07\x12\x1d\n\x15outputWatchdogTimeout\x18\x03 \x01(\x07\"\x17\n\x15\x43onfigurationDescribe\"9\n\x1d\x43onfigurationDescribeResponse\x12\x18\n\x10numberOfChannels\x18\x01 \x01(\x07\"+\n\tSetSingle\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\x07\x12\r\n\x05state\x18\x02 \x01(\x08\"&\n\x06SetAll\x12\x0e\n\x06values\x18\x01 \x01(\x07\x12\x0c\n\x04mask\x18\x02 \x01(\x07\"\x0e\n\x0cSetExitError\"\xa1\x01\n\x12\x46unctionControlSet\x12*\n\x06single\x18\x01 \x01(\x0b\x32\x18.binaryIoTypeA.SetSingleH\x00\x12$\n\x03\x61ll\x18\x02 \x01(\x0b\x32\x15.binaryIoTypeA.SetAllH\x00\x12\x31\n\nexit_error\x18\x03 \x01(\x0b\x32\x1b.binaryIoTypeA.SetExitErrorH\x00\x42\x06\n\x04type\"\x1c\n\tGetSingle\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\x07\"\x16\n\x06GetAll\x12\x0c\n\x04mask\x18\x01 \x01(\x07\"n\n\x12\x46unctionControlGet\x12*\n\x06single\x18\x01 \x01(\x0b\x32\x18.binaryIoTypeA.GetSingleH\x00\x12$\n\x03\x61ll\x18\x02 \x01(\x0b\x32\x15.binaryIoTypeA.GetAllH\x00\x42\x06\n\x04type\" \n\x0eGetAllResponse\x12\x0e\n\x06inputs\x18\x01 \x01(\x07\"3\n\x11GetSingleResponse\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\x07\x12\r\n\x05state\x18\x02 \x01(\x08\"\x86\x01\n\x1a\x46unctionControlGetResponse\x12\x32\n\x06single\x18\x01 \x01(\x0b\x32 .binaryIoTypeA.GetSingleResponseH\x00\x12,\n\x03\x61ll\x18\x02 \x01(\x0b\x32\x1d.binaryIoTypeA.GetAllResponseH\x00\x42\x06\n\x04type\"\x10\n\x0eSetAllResponse\"\x13\n\x11SetSingleResponse\"\x86\x01\n\x1a\x46unctionControlSetResponse\x12\x32\n\x06single\x18\x01 \x01(\x0b\x32 .binaryIoTypeA.SetSingleResponseH\x00\x12,\n\x03\x61ll\x18\x02 \x01(\x0b\x32\x1d.binaryIoTypeA.SetAllResponseH\x00\x42\x06\n\x04type\"/\n\x12StreamControlStart\x12\x19\n\x11\x63hannelFilterMask\x18\x01 \x01(\x07\";\n\x06Sample\x12\x11\n\ttimestamp\x18\x01 \x01(\x06\x12\x0f\n\x07\x63hannel\x18\x02 \x01(\x07\x12\r\n\x05value\x18\x03 \x01(\x08\"4\n\nStreamData\x12&\n\x07samples\x18\x01 \x03(\x0b\x32\x15.binaryIoTypeA.SampleB\x18Z\x16\x62inaryIoTypeA/v1alpha1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'binaryIoTypeA.v1alpha1.binaryIoTypeA_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\026binaryIoTypeA/v1alpha1'
  _globals['_CONFIGURATIONSET']._serialized_start=61
  _globals['_CONFIGURATIONSET']._serialized_end=166
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_start=168
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_end=194
  _globals['_CONFIGURATIONGET']._serialized_start=196
  _globals['_CONFIGURATIONGET']._serialized_end=214
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_start=216
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_end=329
  _globals['_CONFIGURATIONDESCRIBE']._serialized_start=331
  _globals['_CONFIGURATIONDESCRIBE']._serialized_end=354
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_start=356
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_end=413
  _globals['_SETSINGLE']._serialized_start=415
  _globals['_SETSINGLE']._serialized_end=458
  _globals['_SETALL']._serialized_start=460
  _globals['_SETALL']._serialized_end=498
  _globals['_SETEXITERROR']._serialized_start=500
  _globals['_SETEXITERROR']._serialized_end=514
  _globals['_FUNCTIONCONTROLSET']._serialized_start=517
  _globals['_FUNCTIONCONTROLSET']._serialized_end=678
  _globals['_GETSINGLE']._serialized_start=680
  _globals['_GETSINGLE']._serialized_end=708
  _globals['_GETALL']._serialized_start=710
  _globals['_GETALL']._serialized_end=732
  _globals['_FUNCTIONCONTROLGET']._serialized_start=734
  _globals['_FUNCTIONCONTROLGET']._serialized_end=844
  _globals['_GETALLRESPONSE']._serialized_start=846
  _globals['_GETALLRESPONSE']._serialized_end=878
  _globals['_GETSINGLERESPONSE']._serialized_start=880
  _globals['_GETSINGLERESPONSE']._serialized_end=931
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_start=934
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_end=1068
  _globals['_SETALLRESPONSE']._serialized_start=1070
  _globals['_SETALLRESPONSE']._serialized_end=1086
  _globals['_SETSINGLERESPONSE']._serialized_start=1088
  _globals['_SETSINGLERESPONSE']._serialized_end=1107
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_start=1110
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_end=1244
  _globals['_STREAMCONTROLSTART']._serialized_start=1246
  _globals['_STREAMCONTROLSTART']._serialized_end=1293
  _globals['_SAMPLE']._serialized_start=1295
  _globals['_SAMPLE']._serialized_end=1354
  _globals['_STREAMDATA']._serialized_start=1356
  _globals['_STREAMDATA']._serialized_end=1408
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: analogInTypeA/v1alpha1/analogInTypeA.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*analogInTypeA/v1alpha1/analogInTypeA.proto\x12\ranalogInTypeA\"\'\n\x10\x43onfigurationSet\x12\x13\n\x0bsample_rate\x18\x01 \x01(\x07\"\x1a\n\x18\x43onfigurationSetResponse\"\x12\n\x10\x43onfigurationGet\"/\n\x18\x43onfigurationGetResponse\x12\x13\n\x0bsample_rate\x18\x01 \x01(\x07\"\x17\n\x15\x43onfigurationDescribe\"\x1f\n\x1d\x43onfigurationDescribeResponse\"\x14\n\x12\x46unctionControlGet\"\x14\n\x12\x46unctionControlSet\"+\n\x1a\x46unctionControlGetResponse\x12\r\n\x05value\x18\x01 \x01(\x02\"\x1c\n\x1a\x46unctionControlSetResponse\"\x14\n\x12StreamControlStart\"*\n\x06Sample\x12\x11\n\ttimestamp\x18\x01 \x01(\x06\x12\r\n\x05value\x18\x02 \x01(\x02\"4\n\nStreamData\x12&\n\x07samples\x18\x01 \x03(\x0b\x32\x15.analogInTypeA.SampleB\x18Z\x16\x61nalogInTypeA/v1alpha1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'analogInTypeA.v1alpha1.analogInTypeA_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\026analogInTypeA/v1alpha1'
  _globals['_CONFIGURATIONSET']._serialized_start=61
  _globals['_CONFIGURATIONSET']._serialized_end=100
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_start=102
  _globals['_CONFIGURATIONSETRESPONSE']._serialized_end=128
  _globals['_CONFIGURATIONGET']._serialized_start=130
  _globals['_CONFIGURATIONGET']._serialized_end=148
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_start=150
  _globals['_CONFIGURATIONGETRESPONSE']._serialized_end=197
  _globals['_CONFIGURATIONDESCRIBE']._serialized_start=199
  _globals['_CONFIGURATIONDESCRIBE']._serialized_end=222
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_start=224
  _globals['_CONFIGURATIONDESCRIBERESPONSE']._serialized_end=255
  _globals['_FUNCTIONCONTROLGET']._serialized_start=257
  _globals['_FUNCTIONCONTROLGET']._serialized_end=277
  _globals['_FUNCTIONCONTROLSET']._serialized_start=279
  _globals['_FUNCTIONCONTROLSET']._serialized_end=299
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_start=301
  _globals['_FUNCTIONCONTROLGETRESPONSE']._serialized_end=344
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_start=346
  _globals['_FUNCTIONCONTROLSETRESPONSE']._serialized_end=374
  _globals['_STREAMCONTROLSTART']._serialized_start=376
  _globals['_STREAMCONTROLSTART']._serialized_end=396
  _globals['_SAMPLE']._serialized_start=398
  _globals['_SAMPLE']._serialized_end=440
  _globals['_STREAMDATA']._serialized_start=442
  _globals['_STREAMDATA']._serialized_end=494
# @@protoc_insertion_point(module_scope)
